/****************************************************************************
** Meta object code from reading C++ file 'jkqtpbaseplotter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpbaseplotter.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpbaseplotter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15JKQTBasePlotterE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTBasePlotter::qt_create_metaobjectdata<qt_meta_tag_ZN15JKQTBasePlotterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTBasePlotter",
        "beforeExporting",
        "",
        "afterExporting",
        "beforePrinting",
        "afterPrinting",
        "zoomChangedLocally",
        "newxmin",
        "newxmax",
        "newymin",
        "newymax",
        "JKQTBasePlotter*",
        "sender",
        "plotUpdated",
        "plotScalingRecalculated",
        "beforePlotScalingRecalculate",
        "clearGraphs",
        "deleteGraphs",
        "setAllGraphsInvisible",
        "setAllGraphsVisible",
        "setGraphVisible",
        "i",
        "visible",
        "setGraphInvisible",
        "saveData",
        "filename",
        "format",
        "copyData",
        "copyDataMatlab",
        "saveAsCSV",
        "saveAsSemicolonSV",
        "saveAsTabSV",
        "saveAsDIF",
        "saveAsSYLK",
        "saveAsMatlab",
        "saveAsGerExcelCSV",
        "saveAsPixelImage",
        "displayPreview",
        "outputFormat",
        "QSize",
        "outputSizeIncrease",
        "grabPixelImage",
        "QImage",
        "size",
        "showPreview",
        "copyPixelImage",
        "saveAsSVG",
        "saveAsPDF",
        "saveImage",
        "print",
        "QPrinter*",
        "printer",
        "setWidgetWidth",
        "wid",
        "setWidgetSize",
        "width",
        "height",
        "setWidgetHeight",
        "heigh",
        "setPlotBorder",
        "left",
        "right",
        "top",
        "bottom",
        "setPlotBorderLeft",
        "setPlotBorderRight",
        "setPlotBorderTop",
        "setPlotBorderBottom",
        "setX",
        "xminn",
        "xmaxx",
        "affectsSecondaryAxes",
        "setY",
        "yminn",
        "ymaxx",
        "setXY",
        "zoom",
        "setAbsoluteX",
        "setAbsoluteY",
        "setAbsoluteXY",
        "setGrid",
        "val",
        "setGridColor",
        "QColor",
        "color",
        "setMinorGridColor",
        "setGridWidth",
        "__value",
        "setMinorGridWidth",
        "setGridStyle",
        "Qt::PenStyle",
        "setMinorGridStyle",
        "setShowZeroAxes",
        "showX",
        "showY",
        "showXY",
        "zoomToFit",
        "zoomX",
        "zoomY",
        "includeX0",
        "includeY0",
        "scaleX",
        "scaleY",
        "zoomIn",
        "factor",
        "zoomOut",
        "setMaintainAspectRatio",
        "value",
        "setMaintainAxisAspectRatio",
        "setUserSettigsFilename",
        "prefix",
        "setUserSettigsPrefix",
        "getUserSettigsFilename",
        "getUserSettigsPrefix",
        "loadUserSettings",
        "saveUserSettings",
        "setOnlyGraphVisible",
        "setOnlyNthGraphsVisible",
        "start",
        "n",
        "setAspectRatio",
        "setAxisAspectRatio",
        "setUseAntiAliasingForSystem",
        "setUseAntiAliasingForGraphs",
        "setUseAntiAliasingForText",
        "setBackgroundColor",
        "setExportBackgroundColor",
        "setPlotBackgroundColor",
        "setBackgroundBrush",
        "QBrush",
        "setExportBackgroundBrush",
        "setPlotBackgroundBrush",
        "setBackgroundGradient",
        "QGradient",
        "setExportBackgroundGradient",
        "setPlotBackgroundGradient",
        "setBackgroundTexture",
        "QPixmap",
        "setExportBackgroundTexture",
        "setPlotBackgroundTexture",
        "setPlotFrameWidth",
        "setPlotFrameRounding",
        "setPlotFrameColor",
        "col",
        "setPlotFrameVisible",
        "enabled",
        "setShowKey",
        "setKeyPosition",
        "JKQTPKeyPosition",
        "setPlotLabelFontSize",
        "setPlotLabelOffset",
        "setPlotLabelTopBorder",
        "setplotLabelFontName",
        "setPlotLabelColor",
        "setPlotLabel",
        "setDefaultTextColor",
        "setDefaultTextSize",
        "setDefaultTextFontName",
        "setCurrentSaveDirectory",
        "setCurrentFileFormat",
        "setCSVdecimalSeparator",
        "setCSVcommentInitializer",
        "enableDebugShowRegionBoxes",
        "enableDebugShowTextBoxes",
        "updatePreviewLabel",
        "printpreviewPaintRequested",
        "printpreviewPaintRequestedNewPrinter",
        "printpreviewPaintRequestedNewPaintDevice",
        "QPaintDevice*",
        "paintDevice",
        "printpreviewSetZoom",
        "printpreviewSetSizeX",
        "printpreviewSetSizeY",
        "printpreviewSetSizeXNew",
        "printpreviewSetSizeYNew",
        "printpreviewSetMagnification",
        "printpreviewSetMagnificationNew",
        "printpreviewSetAspectRatio",
        "checked",
        "printpreviewSetKeepAbsFontsize",
        "printpreviewToggleMagnification",
        "printpreviewSetLineWidthMultiplier",
        "printpreviewSetFontSizeMultiplier",
        "printpreviewUpdate",
        "exportpreviewPaintRequested",
        "JKQTPEnhancedPainter&",
        "painter",
        "getDataColumnsByUserCheckAll",
        "getDataColumnsByUserCheckNone",
        "getDataColumnsByUserSave",
        "getDataColumnsByUserComboBoxSelected",
        "name",
        "getDataColumnsByUserItemChanged",
        "QListWidgetItem*",
        "widgetitem",
        "showPlotData",
        "synchronizeXAxis",
        "synchronizeYAxis",
        "synchronizeXYAxis"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beforeExporting'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'afterExporting'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beforePrinting'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'afterPrinting'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomChangedLocally'
        QtMocHelpers::SignalData<void(double, double, double, double, JKQTBasePlotter *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'plotUpdated'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'plotScalingRecalculated'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beforePlotScalingRecalculate'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearGraphs'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'clearGraphs'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setAllGraphsInvisible'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAllGraphsVisible'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setGraphVisible'
        QtMocHelpers::SlotData<void(int, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'setGraphVisible'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'setGraphInvisible'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'saveData'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 26 },
        }}),
        // Slot 'saveData'
        QtMocHelpers::SlotData<bool(const QString &)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveData'
        QtMocHelpers::SlotData<bool()>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'copyData'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyDataMatlab'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsCSV'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsCSV'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsSemicolonSV'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsSemicolonSV'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsTabSV'
        QtMocHelpers::SlotData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsTabSV'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsDIF'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsDIF'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsSYLK'
        QtMocHelpers::SlotData<void(const QString &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsSYLK'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsMatlab'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsMatlab'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsGerExcelCSV'
        QtMocHelpers::SlotData<void(const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsGerExcelCSV'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool, const QByteArray &, const QSize &)>(36, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 }, { QMetaType::QByteArray, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool, const QByteArray &)>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 }, { QMetaType::QByteArray, 38 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &)>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool()>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'grabPixelImage'
        QtMocHelpers::SlotData<QImage(QSize, bool)>(41, 2, QMC::AccessPublic, 0x80000000 | 42, {{
            { 0x80000000 | 39, 43 }, { QMetaType::Bool, 44 },
        }}),
        // Slot 'grabPixelImage'
        QtMocHelpers::SlotData<QImage(QSize)>(41, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 42, {{
            { 0x80000000 | 39, 43 },
        }}),
        // Slot 'grabPixelImage'
        QtMocHelpers::SlotData<QImage()>(41, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 42),
        // Slot 'copyPixelImage'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
        // Slot 'copyPixelImage'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(46, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool(const QString &)>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool()>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(47, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool(const QString &)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool()>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(48, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool(const QString &)>(48, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool()>(48, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'print'
        QtMocHelpers::SlotData<void(QPrinter *, bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 51 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void(QPrinter *)>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setWidgetWidth'
        QtMocHelpers::SlotData<void(int)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 53 },
        }}),
        // Slot 'setWidgetSize'
        QtMocHelpers::SlotData<void(int, int)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 55 }, { QMetaType::Int, 56 },
        }}),
        // Slot 'setWidgetSize'
        QtMocHelpers::SlotData<void(QSize)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 43 },
        }}),
        // Slot 'setWidgetHeight'
        QtMocHelpers::SlotData<void(int)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 58 },
        }}),
        // Slot 'setPlotBorder'
        QtMocHelpers::SlotData<void(int, int, int, int)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 60 }, { QMetaType::Int, 61 }, { QMetaType::Int, 62 }, { QMetaType::Int, 63 },
        }}),
        // Slot 'setPlotBorderLeft'
        QtMocHelpers::SlotData<void(int)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 60 },
        }}),
        // Slot 'setPlotBorderRight'
        QtMocHelpers::SlotData<void(int)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 61 },
        }}),
        // Slot 'setPlotBorderTop'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 62 },
        }}),
        // Slot 'setPlotBorderBottom'
        QtMocHelpers::SlotData<void(int)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 63 },
        }}),
        // Slot 'setX'
        QtMocHelpers::SlotData<void(double, double, bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Bool, 71 },
        }}),
        // Slot 'setX'
        QtMocHelpers::SlotData<void(double, double)>(68, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(double, double, bool)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 73 }, { QMetaType::Double, 74 }, { QMetaType::Bool, 71 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(double, double)>(72, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
        }}),
        // Slot 'setXY'
        QtMocHelpers::SlotData<void(double, double, double, double, bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
            { QMetaType::Bool, 71 },
        }}),
        // Slot 'setXY'
        QtMocHelpers::SlotData<void(double, double, double, double)>(75, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
        }}),
        // Slot 'zoom'
        QtMocHelpers::SlotData<void(double, double, double, double, bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
            { QMetaType::Bool, 71 },
        }}),
        // Slot 'zoom'
        QtMocHelpers::SlotData<void(double, double, double, double)>(76, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
        }}),
        // Slot 'setAbsoluteX'
        QtMocHelpers::SlotData<void(double, double)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 },
        }}),
        // Slot 'setAbsoluteY'
        QtMocHelpers::SlotData<void(double, double)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
        }}),
        // Slot 'setAbsoluteXY'
        QtMocHelpers::SlotData<void(double, double, double, double)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 69 }, { QMetaType::Double, 70 }, { QMetaType::Double, 73 }, { QMetaType::Double, 74 },
        }}),
        // Slot 'setGrid'
        QtMocHelpers::SlotData<void(bool)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 81 },
        }}),
        // Slot 'setGridColor'
        QtMocHelpers::SlotData<void(QColor)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 84 },
        }}),
        // Slot 'setMinorGridColor'
        QtMocHelpers::SlotData<void(QColor)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 84 },
        }}),
        // Slot 'setGridWidth'
        QtMocHelpers::SlotData<void(double)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setMinorGridWidth'
        QtMocHelpers::SlotData<void(double)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 90, 87 },
        }}),
        // Slot 'setMinorGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 90, 87 },
        }}),
        // Slot 'setShowZeroAxes'
        QtMocHelpers::SlotData<void(bool, bool)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 93 }, { QMetaType::Bool, 94 },
        }}),
        // Slot 'setShowZeroAxes'
        QtMocHelpers::SlotData<void(bool)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 95 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool, double, double)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 97 }, { QMetaType::Bool, 98 }, { QMetaType::Bool, 99 }, { QMetaType::Bool, 100 },
            { QMetaType::Double, 101 }, { QMetaType::Double, 102 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool, double)>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 97 }, { QMetaType::Bool, 98 }, { QMetaType::Bool, 99 }, { QMetaType::Bool, 100 },
            { QMetaType::Double, 101 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool)>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 97 }, { QMetaType::Bool, 98 }, { QMetaType::Bool, 99 }, { QMetaType::Bool, 100 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool)>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 97 }, { QMetaType::Bool, 98 }, { QMetaType::Bool, 99 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool)>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 97 }, { QMetaType::Bool, 98 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool)>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 97 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void(double)>(103, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 104 },
        }}),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void(double)>(105, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 104 },
        }}),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setMaintainAspectRatio'
        QtMocHelpers::SlotData<void(bool)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 107 },
        }}),
        // Slot 'setMaintainAxisAspectRatio'
        QtMocHelpers::SlotData<void(bool)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 107 },
        }}),
        // Slot 'setUserSettigsFilename'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 110 },
        }}),
        // Slot 'setUserSettigsFilename'
        QtMocHelpers::SlotData<void(const QString &)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'setUserSettigsPrefix'
        QtMocHelpers::SlotData<void(const QString &)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 110 },
        }}),
        // Slot 'getUserSettigsFilename'
        QtMocHelpers::SlotData<QString() const>(112, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getUserSettigsPrefix'
        QtMocHelpers::SlotData<QString() const>(113, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'loadUserSettings'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveUserSettings'
        QtMocHelpers::SlotData<void() const>(115, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOnlyGraphVisible'
        QtMocHelpers::SlotData<void(int)>(116, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'setOnlyNthGraphsVisible'
        QtMocHelpers::SlotData<void(int, int)>(117, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 118 }, { QMetaType::Int, 119 },
        }}),
        // Slot 'setAspectRatio'
        QtMocHelpers::SlotData<void(double)>(120, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setAxisAspectRatio'
        QtMocHelpers::SlotData<void(double)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setUseAntiAliasingForSystem'
        QtMocHelpers::SlotData<void(bool)>(122, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'setUseAntiAliasingForGraphs'
        QtMocHelpers::SlotData<void(bool)>(123, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'setUseAntiAliasingForText'
        QtMocHelpers::SlotData<void(bool)>(124, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'setBackgroundColor'
        QtMocHelpers::SlotData<void(const QColor &)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 87 },
        }}),
        // Slot 'setExportBackgroundColor'
        QtMocHelpers::SlotData<void(const QColor &)>(126, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 87 },
        }}),
        // Slot 'setPlotBackgroundColor'
        QtMocHelpers::SlotData<void(const QColor &)>(127, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 87 },
        }}),
        // Slot 'setBackgroundBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(128, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 129, 87 },
        }}),
        // Slot 'setExportBackgroundBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(130, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 129, 87 },
        }}),
        // Slot 'setPlotBackgroundBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(131, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 129, 87 },
        }}),
        // Slot 'setBackgroundGradient'
        QtMocHelpers::SlotData<void(const QGradient &)>(132, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 133, 87 },
        }}),
        // Slot 'setExportBackgroundGradient'
        QtMocHelpers::SlotData<void(const QGradient &)>(134, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 133, 87 },
        }}),
        // Slot 'setPlotBackgroundGradient'
        QtMocHelpers::SlotData<void(const QGradient &)>(135, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 133, 87 },
        }}),
        // Slot 'setBackgroundTexture'
        QtMocHelpers::SlotData<void(const QPixmap &)>(136, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 137, 87 },
        }}),
        // Slot 'setExportBackgroundTexture'
        QtMocHelpers::SlotData<void(const QPixmap &)>(138, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 137, 87 },
        }}),
        // Slot 'setPlotBackgroundTexture'
        QtMocHelpers::SlotData<void(const QPixmap &)>(139, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 137, 87 },
        }}),
        // Slot 'setBackgroundTexture'
        QtMocHelpers::SlotData<void(const QImage &)>(136, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 87 },
        }}),
        // Slot 'setExportBackgroundTexture'
        QtMocHelpers::SlotData<void(const QImage &)>(138, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 87 },
        }}),
        // Slot 'setPlotBackgroundTexture'
        QtMocHelpers::SlotData<void(const QImage &)>(139, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 87 },
        }}),
        // Slot 'setPlotFrameWidth'
        QtMocHelpers::SlotData<void(double)>(140, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setPlotFrameRounding'
        QtMocHelpers::SlotData<void(double)>(141, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setPlotFrameColor'
        QtMocHelpers::SlotData<void(QColor)>(142, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 143 },
        }}),
        // Slot 'setPlotFrameVisible'
        QtMocHelpers::SlotData<void(bool)>(144, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 145 },
        }}),
        // Slot 'setShowKey'
        QtMocHelpers::SlotData<void(bool)>(146, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'setKeyPosition'
        QtMocHelpers::SlotData<void(const JKQTPKeyPosition &)>(147, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 148, 87 },
        }}),
        // Slot 'setPlotLabelFontSize'
        QtMocHelpers::SlotData<void(double)>(149, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setPlotLabelOffset'
        QtMocHelpers::SlotData<void(double)>(150, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setPlotLabelTopBorder'
        QtMocHelpers::SlotData<void(double)>(151, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setplotLabelFontName'
        QtMocHelpers::SlotData<void(const QString &)>(152, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setPlotLabelColor'
        QtMocHelpers::SlotData<void(QColor)>(153, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 87 },
        }}),
        // Slot 'setPlotLabel'
        QtMocHelpers::SlotData<void(const QString &)>(154, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setDefaultTextColor'
        QtMocHelpers::SlotData<void(QColor)>(155, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 83, 87 },
        }}),
        // Slot 'setDefaultTextSize'
        QtMocHelpers::SlotData<void(double)>(156, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 87 },
        }}),
        // Slot 'setDefaultTextFontName'
        QtMocHelpers::SlotData<void(const QString &)>(157, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setCurrentSaveDirectory'
        QtMocHelpers::SlotData<void(const QString &)>(158, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setCurrentFileFormat'
        QtMocHelpers::SlotData<void(const QString &)>(159, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setCSVdecimalSeparator'
        QtMocHelpers::SlotData<void(const QString &)>(160, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'setCSVcommentInitializer'
        QtMocHelpers::SlotData<void(const QString &)>(161, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 87 },
        }}),
        // Slot 'enableDebugShowRegionBoxes'
        QtMocHelpers::SlotData<void(bool)>(162, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 145 },
        }}),
        // Slot 'enableDebugShowRegionBoxes'
        QtMocHelpers::SlotData<void()>(162, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'enableDebugShowTextBoxes'
        QtMocHelpers::SlotData<void(bool)>(163, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 145 },
        }}),
        // Slot 'enableDebugShowTextBoxes'
        QtMocHelpers::SlotData<void()>(163, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'updatePreviewLabel'
        QtMocHelpers::SlotData<void()>(164, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'printpreviewPaintRequested'
        QtMocHelpers::SlotData<void(QPrinter *)>(165, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'printpreviewPaintRequestedNewPrinter'
        QtMocHelpers::SlotData<void(QPrinter *)>(166, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 50, 51 },
        }}),
        // Slot 'printpreviewPaintRequestedNewPaintDevice'
        QtMocHelpers::SlotData<void(QPaintDevice *)>(167, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 168, 169 },
        }}),
        // Slot 'printpreviewSetZoom'
        QtMocHelpers::SlotData<void(double)>(170, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetSizeX'
        QtMocHelpers::SlotData<void(double)>(171, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetSizeY'
        QtMocHelpers::SlotData<void(double)>(172, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetSizeXNew'
        QtMocHelpers::SlotData<void(double)>(173, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetSizeYNew'
        QtMocHelpers::SlotData<void(double)>(174, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetMagnification'
        QtMocHelpers::SlotData<void(double)>(175, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetMagnificationNew'
        QtMocHelpers::SlotData<void(double)>(176, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetAspectRatio'
        QtMocHelpers::SlotData<void(bool)>(177, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 178 },
        }}),
        // Slot 'printpreviewSetKeepAbsFontsize'
        QtMocHelpers::SlotData<void(bool)>(179, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 178 },
        }}),
        // Slot 'printpreviewToggleMagnification'
        QtMocHelpers::SlotData<void(bool)>(180, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 178 },
        }}),
        // Slot 'printpreviewSetLineWidthMultiplier'
        QtMocHelpers::SlotData<void(double)>(181, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewSetFontSizeMultiplier'
        QtMocHelpers::SlotData<void(double)>(182, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 107 },
        }}),
        // Slot 'printpreviewUpdate'
        QtMocHelpers::SlotData<void()>(183, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'exportpreviewPaintRequested'
        QtMocHelpers::SlotData<void(JKQTPEnhancedPainter &, QSize)>(184, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 185, 186 }, { 0x80000000 | 39, 43 },
        }}),
        // Slot 'getDataColumnsByUserCheckAll'
        QtMocHelpers::SlotData<void()>(187, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'getDataColumnsByUserCheckNone'
        QtMocHelpers::SlotData<void()>(188, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'getDataColumnsByUserSave'
        QtMocHelpers::SlotData<void()>(189, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'getDataColumnsByUserComboBoxSelected'
        QtMocHelpers::SlotData<void(const QString &)>(190, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 191 },
        }}),
        // Slot 'getDataColumnsByUserItemChanged'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(192, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 193, 194 },
        }}),
        // Slot 'showPlotData'
        QtMocHelpers::SlotData<void()>(195, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'synchronizeXAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTBasePlotter *)>(196, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'synchronizeYAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTBasePlotter *)>(197, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'synchronizeXYAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTBasePlotter *)>(198, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTBasePlotter, qt_meta_tag_ZN15JKQTBasePlotterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTBasePlotter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTBasePlotterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTBasePlotterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15JKQTBasePlotterE_t>.metaTypes,
    nullptr
} };

void JKQTBasePlotter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTBasePlotter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beforeExporting(); break;
        case 1: _t->afterExporting(); break;
        case 2: _t->beforePrinting(); break;
        case 3: _t->afterPrinting(); break;
        case 4: _t->zoomChangedLocally((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTBasePlotter*>>(_a[5]))); break;
        case 5: _t->plotUpdated(); break;
        case 6: _t->plotScalingRecalculated(); break;
        case 7: _t->beforePlotScalingRecalculate(); break;
        case 8: _t->clearGraphs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->clearGraphs(); break;
        case 10: _t->setAllGraphsInvisible(); break;
        case 11: _t->setAllGraphsVisible(); break;
        case 12: _t->setGraphVisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->setGraphVisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->setGraphInvisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: { bool _r = _t->saveData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->saveData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->saveData();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->copyData(); break;
        case 19: _t->copyDataMatlab(); break;
        case 20: _t->saveAsCSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->saveAsCSV(); break;
        case 22: _t->saveAsSemicolonSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->saveAsSemicolonSV(); break;
        case 24: _t->saveAsTabSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->saveAsTabSV(); break;
        case 26: _t->saveAsDIF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->saveAsDIF(); break;
        case 28: _t->saveAsSYLK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->saveAsSYLK(); break;
        case 30: _t->saveAsMatlab((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 31: _t->saveAsMatlab(); break;
        case 32: _t->saveAsGerExcelCSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->saveAsGerExcelCSV(); break;
        case 34: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->saveAsPixelImage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { QImage _r = _t->grabPixelImage((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 40: { QImage _r = _t->grabPixelImage((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 41: { QImage _r = _t->grabPixelImage();
            if (_a[0]) *reinterpret_cast<QImage*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->copyPixelImage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->copyPixelImage(); break;
        case 44: { bool _r = _t->saveAsSVG((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 45: { bool _r = _t->saveAsSVG((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 46: { bool _r = _t->saveAsSVG();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 47: { bool _r = _t->saveAsPDF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 48: { bool _r = _t->saveAsPDF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 49: { bool _r = _t->saveAsPDF();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 50: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 51: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 52: { bool _r = _t->saveImage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->print((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 54: _t->print((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 55: _t->print(); break;
        case 56: _t->setWidgetWidth((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 57: _t->setWidgetSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 58: _t->setWidgetSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 59: _t->setWidgetHeight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 60: _t->setPlotBorder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 61: _t->setPlotBorderLeft((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 62: _t->setPlotBorderRight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 63: _t->setPlotBorderTop((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 64: _t->setPlotBorderBottom((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 65: _t->setX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 66: _t->setX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 67: _t->setY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 68: _t->setY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 69: _t->setXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 70: _t->setXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 71: _t->zoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 72: _t->zoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 73: _t->setAbsoluteX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 74: _t->setAbsoluteY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 75: _t->setAbsoluteXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 76: _t->setGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: _t->setGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 78: _t->setMinorGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 79: _t->setGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 80: _t->setMinorGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 81: _t->setGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 82: _t->setMinorGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 83: _t->setShowZeroAxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 84: _t->setShowZeroAxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6]))); break;
        case 86: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5]))); break;
        case 87: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 88: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 89: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 90: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 91: _t->zoomToFit(); break;
        case 92: _t->zoomIn((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 93: _t->zoomIn(); break;
        case 94: _t->zoomOut((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 95: _t->zoomOut(); break;
        case 96: _t->setMaintainAspectRatio((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 97: _t->setMaintainAxisAspectRatio((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 98: _t->setUserSettigsFilename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 99: _t->setUserSettigsFilename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 100: _t->setUserSettigsPrefix((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 101: { QString _r = _t->getUserSettigsFilename();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 102: { QString _r = _t->getUserSettigsPrefix();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 103: _t->loadUserSettings(); break;
        case 104: _t->saveUserSettings(); break;
        case 105: _t->setOnlyGraphVisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 106: _t->setOnlyNthGraphsVisible((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 107: _t->setAspectRatio((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 108: _t->setAxisAspectRatio((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 109: _t->setUseAntiAliasingForSystem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 110: _t->setUseAntiAliasingForGraphs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 111: _t->setUseAntiAliasingForText((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 112: _t->setBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 113: _t->setExportBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 114: _t->setPlotBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 115: _t->setBackgroundBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 116: _t->setExportBackgroundBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 117: _t->setPlotBackgroundBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 118: _t->setBackgroundGradient((*reinterpret_cast<std::add_pointer_t<QGradient>>(_a[1]))); break;
        case 119: _t->setExportBackgroundGradient((*reinterpret_cast<std::add_pointer_t<QGradient>>(_a[1]))); break;
        case 120: _t->setPlotBackgroundGradient((*reinterpret_cast<std::add_pointer_t<QGradient>>(_a[1]))); break;
        case 121: _t->setBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 122: _t->setExportBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 123: _t->setPlotBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[1]))); break;
        case 124: _t->setBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 125: _t->setExportBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 126: _t->setPlotBackgroundTexture((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 127: _t->setPlotFrameWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 128: _t->setPlotFrameRounding((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 129: _t->setPlotFrameColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 130: _t->setPlotFrameVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 131: _t->setShowKey((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 132: _t->setKeyPosition((*reinterpret_cast<std::add_pointer_t<JKQTPKeyPosition>>(_a[1]))); break;
        case 133: _t->setPlotLabelFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 134: _t->setPlotLabelOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 135: _t->setPlotLabelTopBorder((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 136: _t->setplotLabelFontName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 137: _t->setPlotLabelColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 138: _t->setPlotLabel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 139: _t->setDefaultTextColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 140: _t->setDefaultTextSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 141: _t->setDefaultTextFontName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 142: _t->setCurrentSaveDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 143: _t->setCurrentFileFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 144: _t->setCSVdecimalSeparator((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 145: _t->setCSVcommentInitializer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 146: _t->enableDebugShowRegionBoxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 147: _t->enableDebugShowRegionBoxes(); break;
        case 148: _t->enableDebugShowTextBoxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 149: _t->enableDebugShowTextBoxes(); break;
        case 150: _t->updatePreviewLabel(); break;
        case 151: _t->printpreviewPaintRequested((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 152: _t->printpreviewPaintRequestedNewPrinter((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 153: _t->printpreviewPaintRequestedNewPaintDevice((*reinterpret_cast<std::add_pointer_t<QPaintDevice*>>(_a[1]))); break;
        case 154: _t->printpreviewSetZoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 155: _t->printpreviewSetSizeX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 156: _t->printpreviewSetSizeY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 157: _t->printpreviewSetSizeXNew((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 158: _t->printpreviewSetSizeYNew((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 159: _t->printpreviewSetMagnification((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 160: _t->printpreviewSetMagnificationNew((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 161: _t->printpreviewSetAspectRatio((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 162: _t->printpreviewSetKeepAbsFontsize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 163: _t->printpreviewToggleMagnification((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 164: _t->printpreviewSetLineWidthMultiplier((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 165: _t->printpreviewSetFontSizeMultiplier((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 166: _t->printpreviewUpdate(); break;
        case 167: _t->exportpreviewPaintRequested((*reinterpret_cast<std::add_pointer_t<JKQTPEnhancedPainter&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 168: _t->getDataColumnsByUserCheckAll(); break;
        case 169: _t->getDataColumnsByUserCheckNone(); break;
        case 170: _t->getDataColumnsByUserSave(); break;
        case 171: _t->getDataColumnsByUserComboBoxSelected((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 172: _t->getDataColumnsByUserItemChanged((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 173: _t->showPlotData(); break;
        case 174: _t->synchronizeXAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTBasePlotter*>>(_a[5]))); break;
        case 175: _t->synchronizeYAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTBasePlotter*>>(_a[5]))); break;
        case 176: _t->synchronizeXYAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTBasePlotter*>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTBasePlotter* >(); break;
            }
            break;
        case 174:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTBasePlotter* >(); break;
            }
            break;
        case 175:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTBasePlotter* >(); break;
            }
            break;
        case 176:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTBasePlotter* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::beforeExporting, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::afterExporting, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::beforePrinting, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::afterPrinting, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)(double , double , double , double , JKQTBasePlotter * )>(_a, &JKQTBasePlotter::zoomChangedLocally, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::plotUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::plotScalingRecalculated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTBasePlotter::*)()>(_a, &JKQTBasePlotter::beforePlotScalingRecalculate, 7))
            return;
    }
}

const QMetaObject *JKQTBasePlotter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTBasePlotter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTBasePlotterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JKQTBasePlotter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 177)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 177;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 177)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 177;
    }
    return _id;
}

// SIGNAL 0
void JKQTBasePlotter::beforeExporting()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void JKQTBasePlotter::afterExporting()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void JKQTBasePlotter::beforePrinting()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void JKQTBasePlotter::afterPrinting()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void JKQTBasePlotter::zoomChangedLocally(double _t1, double _t2, double _t3, double _t4, JKQTBasePlotter * _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 5
void JKQTBasePlotter::plotUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void JKQTBasePlotter::plotScalingRecalculated()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void JKQTBasePlotter::beforePlotScalingRecalculate()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
