/****************************************************************************
** Meta object code from reading C++ file 'jkqtfastplotter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtfastplotter/jkqtfastplotter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtfastplotter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15JKQTFastPlotterE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFastPlotter::qt_create_metaobjectdata<qt_meta_tag_ZN15JKQTFastPlotterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFastPlotter",
        "plotterSizesChanged",
        "",
        "replotting",
        "clicked",
        "x",
        "y",
        "Qt::KeyboardModifiers",
        "modifiers",
        "doubleClicked",
        "mouseMoved",
        "mouseDragged",
        "x_start",
        "y_start",
        "x_end",
        "y_end",
        "mouseDragFinished",
        "copyImage",
        "redrawPlot",
        "redrawPlotImmediate",
        "updateData",
        "updateDataImmediate",
        "setXMin",
        "value",
        "setXMax",
        "setYMin",
        "setYMax"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'plotterSizesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replotting'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(double, double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
        }}),
        // Signal 'clicked'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { QMetaType::Double, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
        }}),
        // Signal 'doubleClicked'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { QMetaType::Double, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'mouseMoved'
        QtMocHelpers::SignalData<void(double, double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
        }}),
        // Signal 'mouseDragged'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 }, { QMetaType::Double, 15 },
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'mouseDragFinished'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 }, { QMetaType::Double, 15 },
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'copyImage'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redrawPlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redrawPlotImmediate'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateData'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateDataImmediate'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setXMin'
        QtMocHelpers::SlotData<void(double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
        // Slot 'setXMax'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
        // Slot 'setYMin'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
        // Slot 'setYMax'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFastPlotter, qt_meta_tag_ZN15JKQTFastPlotterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFastPlotter::staticMetaObject = { {
    QMetaObject::SuperData::link<QOpenGLWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFastPlotterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFastPlotterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15JKQTFastPlotterE_t>.metaTypes,
    nullptr
} };

void JKQTFastPlotter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFastPlotter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->plotterSizesChanged(); break;
        case 1: _t->replotting(); break;
        case 2: _t->clicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 3: _t->clicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3]))); break;
        case 4: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: _t->doubleClicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3]))); break;
        case 6: _t->mouseMoved((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 7: _t->mouseDragged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 8: _t->mouseDragFinished((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 9: _t->copyImage(); break;
        case 10: _t->redrawPlot(); break;
        case 11: _t->redrawPlotImmediate(); break;
        case 12: _t->updateData(); break;
        case 13: _t->updateDataImmediate(); break;
        case 14: _t->setXMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->setXMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->setYMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->setYMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)()>(_a, &JKQTFastPlotter::plotterSizesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)()>(_a, &JKQTFastPlotter::replotting, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double )>(_a, &JKQTFastPlotter::clicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double , Qt::KeyboardModifiers )>(_a, &JKQTFastPlotter::clicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double )>(_a, &JKQTFastPlotter::doubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double , Qt::KeyboardModifiers )>(_a, &JKQTFastPlotter::doubleClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double )>(_a, &JKQTFastPlotter::mouseMoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTFastPlotter::mouseDragged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTFastPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTFastPlotter::mouseDragFinished, 8))
            return;
    }
}

const QMetaObject *JKQTFastPlotter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFastPlotter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFastPlotterE_t>.strings))
        return static_cast<void*>(this);
    return QOpenGLWidget::qt_metacast(_clname);
}

int JKQTFastPlotter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QOpenGLWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void JKQTFastPlotter::plotterSizesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void JKQTFastPlotter::replotting()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void JKQTFastPlotter::clicked(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void JKQTFastPlotter::clicked(double _t1, double _t2, Qt::KeyboardModifiers _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void JKQTFastPlotter::doubleClicked(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void JKQTFastPlotter::doubleClicked(double _t1, double _t2, Qt::KeyboardModifiers _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void JKQTFastPlotter::mouseMoved(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void JKQTFastPlotter::mouseDragged(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 8
void JKQTFastPlotter::mouseDragFinished(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3, _t4, _t5);
}
namespace {
struct qt_meta_tag_ZN10JKQTFPPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPPlot::qt_create_metaobjectdata<qt_meta_tag_ZN10JKQTFPPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPPlot",
        "setVisible",
        "",
        "visible"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setVisible'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPPlot, qt_meta_tag_ZN10JKQTFPPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTFPPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTFPPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10JKQTFPPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPPlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTFPPlotE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JKQTFPPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16JKQTFPVCrossPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPVCrossPlot::qt_create_metaobjectdata<qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPVCrossPlot"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPVCrossPlot, qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPVCrossPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPLinePlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPVCrossPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPVCrossPlot *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTFPVCrossPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPVCrossPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPVCrossPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPLinePlot::qt_metacast(_clname);
}

int JKQTFPVCrossPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPLinePlot::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14JKQTFPVBarPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPVBarPlot::qt_create_metaobjectdata<qt_meta_tag_ZN14JKQTFPVBarPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPVBarPlot"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPVBarPlot, qt_meta_tag_ZN14JKQTFPVBarPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPVBarPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPLinePlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTFPVBarPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTFPVBarPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14JKQTFPVBarPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPVBarPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPVBarPlot *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTFPVBarPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPVBarPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTFPVBarPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPLinePlot::qt_metacast(_clname);
}

int JKQTFPVBarPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPLinePlot::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16JKQTFPXRangePlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPXRangePlot::qt_create_metaobjectdata<qt_meta_tag_ZN16JKQTFPXRangePlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPXRangePlot",
        "setCenterline",
        "",
        "centerline",
        "setXmin",
        "xmin",
        "setXmax",
        "xmax"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setCenterline'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setXmin'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'setXmax'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setXmin'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setXmax'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPXRangePlot, qt_meta_tag_ZN16JKQTFPXRangePlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPXRangePlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPXRangePlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPXRangePlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JKQTFPXRangePlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPXRangePlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPXRangePlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setCenterline((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setXmin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setXmax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setXmin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setXmax((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPXRangePlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPXRangePlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPXRangePlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPXRangePlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16JKQTFPYRangePlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPYRangePlot::qt_create_metaobjectdata<qt_meta_tag_ZN16JKQTFPYRangePlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPYRangePlot",
        "setCenterline",
        "",
        "centerline",
        "setYmin",
        "xmin",
        "setYmax",
        "xmax"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setCenterline'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setYmin'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'setYmax'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setYmin'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setYmax'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPYRangePlot, qt_meta_tag_ZN16JKQTFPYRangePlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPYRangePlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPYRangePlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPYRangePlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JKQTFPYRangePlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPYRangePlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPYRangePlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setCenterline((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setYmin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setYmax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setYmin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setYmax((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPYRangePlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPYRangePlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPYRangePlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPYRangePlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16JKQTFPQImagePlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPQImagePlot::qt_create_metaobjectdata<qt_meta_tag_ZN16JKQTFPQImagePlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPQImagePlot"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPQImagePlot, qt_meta_tag_ZN16JKQTFPQImagePlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPQImagePlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPQImagePlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPQImagePlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JKQTFPQImagePlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPQImagePlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPQImagePlot *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTFPQImagePlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPQImagePlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTFPQImagePlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPQImagePlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15JKQTFPimagePlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPimagePlot::qt_create_metaobjectdata<qt_meta_tag_ZN15JKQTFPimagePlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPimagePlot",
        "setRotation",
        "",
        "rotation",
        "setPalette",
        "palette",
        "JKQTFPColorPalette",
        "setColorMin",
        "uint32_t",
        "colorMin",
        "setColorMax",
        "colorMax"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setRotation'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setPalette'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setPalette'
        QtMocHelpers::SlotData<void(JKQTFPColorPalette)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 5 },
        }}),
        // Slot 'setColorMin'
        QtMocHelpers::SlotData<void(uint32_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setColorMax'
        QtMocHelpers::SlotData<void(uint32_t)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 11 },
        }}),
        // Slot 'setColorMin'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'setColorMax'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setColorMin'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'setColorMax'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPimagePlot, qt_meta_tag_ZN15JKQTFPimagePlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPimagePlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFPimagePlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFPimagePlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15JKQTFPimagePlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPimagePlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPimagePlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setRotation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setPalette((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setPalette((*reinterpret_cast<std::add_pointer_t<JKQTFPColorPalette>>(_a[1]))); break;
        case 3: _t->setColorMin((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 4: _t->setColorMax((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 5: _t->setColorMin((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setColorMax((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setColorMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setColorMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPimagePlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPimagePlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15JKQTFPimagePlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPimagePlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPRGBImageOverlayPlot::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPRGBImageOverlayPlot",
        "setRotation",
        "",
        "rotation",
        "setColorMinRed",
        "uint32_t",
        "colorMin",
        "setColorMaxRed",
        "colorMax",
        "setColorMinGreen",
        "setColorMaxGreen",
        "setColorMinBlue",
        "setColorMaxBlue"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setRotation'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setColorMinRed'
        QtMocHelpers::SlotData<void(uint32_t)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setColorMaxRed'
        QtMocHelpers::SlotData<void(uint32_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Slot 'setColorMinRed'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setColorMaxRed'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setColorMinRed'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setColorMaxRed'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setColorMinGreen'
        QtMocHelpers::SlotData<void(uint32_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setColorMaxGreen'
        QtMocHelpers::SlotData<void(uint32_t)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Slot 'setColorMinGreen'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setColorMaxGreen'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setColorMinGreen'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setColorMaxGreen'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setColorMinBlue'
        QtMocHelpers::SlotData<void(uint32_t)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setColorMaxBlue'
        QtMocHelpers::SlotData<void(uint32_t)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
        // Slot 'setColorMinBlue'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setColorMaxBlue'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setColorMinBlue'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setColorMaxBlue'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPRGBImageOverlayPlot, qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPRGBImageOverlayPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPRGBImageOverlayPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPRGBImageOverlayPlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setRotation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setColorMinRed((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 2: _t->setColorMaxRed((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 3: _t->setColorMinRed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setColorMaxRed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setColorMinRed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->setColorMaxRed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setColorMinGreen((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 8: _t->setColorMaxGreen((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 9: _t->setColorMinGreen((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->setColorMaxGreen((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setColorMinGreen((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->setColorMaxGreen((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->setColorMinBlue((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 14: _t->setColorMaxBlue((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 15: _t->setColorMinBlue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->setColorMaxBlue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->setColorMinBlue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->setColorMaxBlue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPRGBImageOverlayPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPRGBImageOverlayPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTFPRGBImageOverlayPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPRGBImageOverlayPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPimageOverlayPlot::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPimageOverlayPlot",
        "setRotation",
        "",
        "rotation",
        "setColor",
        "QColor",
        "color"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setRotation'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPimageOverlayPlot, qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPimageOverlayPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPimageOverlayPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPimageOverlayPlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setRotation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPimageOverlayPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPimageOverlayPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTFPimageOverlayPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPimageOverlayPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPQScaleBarXPlot::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPQScaleBarXPlot",
        "setPosition",
        "",
        "pos"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setPosition'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPQScaleBarXPlot, qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPQScaleBarXPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPQScaleBarXPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPQScaleBarXPlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setPosition((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTFPQScaleBarXPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPQScaleBarXPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTFPQScaleBarXPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPQScaleBarXPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFPQOverlayLinearGridPlot::qt_create_metaobjectdata<qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFPQOverlayLinearGridPlot"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFPQOverlayLinearGridPlot, qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFPQOverlayLinearGridPlot::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTFPPlot::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>.metaTypes,
    nullptr
} };

void JKQTFPQOverlayLinearGridPlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTFPQOverlayLinearGridPlot *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTFPQOverlayLinearGridPlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTFPQOverlayLinearGridPlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTFPQOverlayLinearGridPlotE_t>.strings))
        return static_cast<void*>(this);
    return JKQTFPPlot::qt_metacast(_clname);
}

int JKQTFPQOverlayLinearGridPlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTFPPlot::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
