/****************************************************************************
** Meta object code from reading C++ file 'jkqtpcoordinateaxes.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpcoordinateaxes.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpcoordinateaxes.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19JKQTPCoordinateAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPCoordinateAxis::qt_create_metaobjectdata<qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPCoordinateAxis",
        "setRange",
        "",
        "amin",
        "amax",
        "setMin",
        "setMax",
        "setRangeFixed",
        "fixed",
        "setAbsoluteRange",
        "aamin",
        "aamax",
        "setNoAbsoluteRange",
        "setAxisMinWidth",
        "__value",
        "setAutoAxisSpacing",
        "setMinorTickLabelsEnabled",
        "setLogAxis",
        "setLogAxisBase",
        "setUserTickSpacing",
        "setUserLogTickSpacing",
        "setTickLabelType",
        "JKQTPCALabelType",
        "setTickMode",
        "JKQTPLabelTickMode",
        "setAxisLabel",
        "setTickUnitName",
        "setTickUnitFactor",
        "setTickUnit",
        "factor",
        "name",
        "setTickUnitPi",
        "resetTickUnit",
        "setLabelPosition",
        "JKQTPLabelPosition",
        "setLabelFontSize",
        "setTickTimeFormat",
        "setTickDateFormat",
        "setTickDateTimeFormat",
        "setTickPrintfFormat",
        "setTickLabelFontSize",
        "setMinorTickLabelFontSize",
        "setMinorTickLabelFullNumber",
        "setMinTicks",
        "setMinorTicks",
        "setTickOutsideLength",
        "setMinorTickOutsideLength",
        "setTickInsideLength",
        "setMinorTickInsideLength",
        "setAxisColor",
        "QColor",
        "setShowZeroAxis",
        "setInverted",
        "setGridColor",
        "alpha",
        "setMinorGridColor",
        "setGridWidth",
        "setGridStyle",
        "Qt::PenStyle",
        "setMinorGridWidth",
        "setMinorGridStyle",
        "setDrawMode1",
        "JKQTPCADrawMode",
        "setDrawMode0",
        "setDrawMode2",
        "setMinorTickWidth",
        "setTickWidth",
        "setLineWidth",
        "setLineWidthZeroAxis",
        "setTickLabelDistance",
        "setLabelDistance",
        "setLabelDigits",
        "setDrawGrid",
        "setDrawMinorGrid",
        "setTickLabelAngle",
        "setArrowSizeFactor",
        "f",
        "setLabelColor",
        "c",
        "setMinorTickColor",
        "setMinorTickLabelColor",
        "setTickColor",
        "setTickLabelColor",
        "setColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setRange'
        QtMocHelpers::SlotData<void(double, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Slot 'setMin'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setMax'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 4 },
        }}),
        // Slot 'setRangeFixed'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'setAbsoluteRange'
        QtMocHelpers::SlotData<void(double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'setNoAbsoluteRange'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAxisMinWidth'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setAutoAxisSpacing'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setMinorTickLabelsEnabled'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setLogAxis'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setLogAxisBase'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setUserTickSpacing'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setUserLogTickSpacing'
        QtMocHelpers::SlotData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickLabelType'
        QtMocHelpers::SlotData<void(JKQTPCALabelType)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 14 },
        }}),
        // Slot 'setTickMode'
        QtMocHelpers::SlotData<void(JKQTPLabelTickMode)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 14 },
        }}),
        // Slot 'setTickMode'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setAxisLabel'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickUnitName'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickUnitFactor'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickUnit'
        QtMocHelpers::SlotData<void(double, const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 29 }, { QMetaType::QString, 30 },
        }}),
        // Slot 'setTickUnitPi'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetTickUnit'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLabelPosition'
        QtMocHelpers::SlotData<void(JKQTPLabelPosition)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 14 },
        }}),
        // Slot 'setLabelFontSize'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickTimeFormat'
        QtMocHelpers::SlotData<void(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickDateFormat'
        QtMocHelpers::SlotData<void(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickDateTimeFormat'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickPrintfFormat'
        QtMocHelpers::SlotData<void(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'setTickLabelFontSize'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setMinorTickLabelFontSize'
        QtMocHelpers::SlotData<void(double)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setMinorTickLabelFullNumber'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setMinTicks'
        QtMocHelpers::SlotData<void(unsigned int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 14 },
        }}),
        // Slot 'setMinorTicks'
        QtMocHelpers::SlotData<void(unsigned int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 14 },
        }}),
        // Slot 'setMinorTicks'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setTickOutsideLength'
        QtMocHelpers::SlotData<void(double)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setMinorTickOutsideLength'
        QtMocHelpers::SlotData<void(double)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickInsideLength'
        QtMocHelpers::SlotData<void(double)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setMinorTickInsideLength'
        QtMocHelpers::SlotData<void(double)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setAxisColor'
        QtMocHelpers::SlotData<void(const QColor &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 14 },
        }}),
        // Slot 'setShowZeroAxis'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setInverted'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setGridColor'
        QtMocHelpers::SlotData<void(const QColor &)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 14 },
        }}),
        // Slot 'setGridColor'
        QtMocHelpers::SlotData<void(const QColor &, double)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 14 }, { QMetaType::Double, 54 },
        }}),
        // Slot 'setMinorGridColor'
        QtMocHelpers::SlotData<void(const QColor &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 14 },
        }}),
        // Slot 'setMinorGridColor'
        QtMocHelpers::SlotData<void(const QColor &, double)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 14 }, { QMetaType::Double, 54 },
        }}),
        // Slot 'setGridWidth'
        QtMocHelpers::SlotData<void(double)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 58, 14 },
        }}),
        // Slot 'setMinorGridWidth'
        QtMocHelpers::SlotData<void(double)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setMinorGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 58, 14 },
        }}),
        // Slot 'setDrawMode1'
        QtMocHelpers::SlotData<void(JKQTPCADrawMode)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 14 },
        }}),
        // Slot 'setDrawMode0'
        QtMocHelpers::SlotData<void(JKQTPCADrawMode)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 14 },
        }}),
        // Slot 'setDrawMode2'
        QtMocHelpers::SlotData<void(JKQTPCADrawMode)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 14 },
        }}),
        // Slot 'setMinorTickWidth'
        QtMocHelpers::SlotData<void(double)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickWidth'
        QtMocHelpers::SlotData<void(double)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setLineWidth'
        QtMocHelpers::SlotData<void(double)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setLineWidthZeroAxis'
        QtMocHelpers::SlotData<void(double)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setTickLabelDistance'
        QtMocHelpers::SlotData<void(double)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setLabelDistance'
        QtMocHelpers::SlotData<void(double)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setLabelDigits'
        QtMocHelpers::SlotData<void(int)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setDrawGrid'
        QtMocHelpers::SlotData<void(bool)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setDrawMinorGrid'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setTickLabelAngle'
        QtMocHelpers::SlotData<void(double)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setArrowSizeFactor'
        QtMocHelpers::SlotData<void(double)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 76 },
        }}),
        // Slot 'setLabelColor'
        QtMocHelpers::SlotData<void(QColor)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
        // Slot 'setMinorTickColor'
        QtMocHelpers::SlotData<void(QColor)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
        // Slot 'setMinorTickLabelColor'
        QtMocHelpers::SlotData<void(QColor)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
        // Slot 'setTickColor'
        QtMocHelpers::SlotData<void(QColor)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
        // Slot 'setTickLabelColor'
        QtMocHelpers::SlotData<void(QColor)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 50, 78 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPCoordinateAxis, qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPCoordinateAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>.metaTypes,
    nullptr
} };

void JKQTPCoordinateAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPCoordinateAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setRange((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: _t->setMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setRangeFixed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->setAbsoluteRange((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: _t->setNoAbsoluteRange(); break;
        case 6: _t->setAxisMinWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setAutoAxisSpacing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setMinorTickLabelsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setLogAxis((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setLogAxisBase((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->setUserTickSpacing((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->setUserLogTickSpacing((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->setTickLabelType((*reinterpret_cast<std::add_pointer_t<JKQTPCALabelType>>(_a[1]))); break;
        case 14: _t->setTickMode((*reinterpret_cast<std::add_pointer_t<JKQTPLabelTickMode>>(_a[1]))); break;
        case 15: _t->setTickMode((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->setAxisLabel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->setTickUnitName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->setTickUnitFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->setTickUnit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->setTickUnitPi(); break;
        case 21: _t->resetTickUnit(); break;
        case 22: _t->setLabelPosition((*reinterpret_cast<std::add_pointer_t<JKQTPLabelPosition>>(_a[1]))); break;
        case 23: _t->setLabelFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->setTickTimeFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->setTickDateFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->setTickDateTimeFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->setTickPrintfFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->setTickLabelFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->setMinorTickLabelFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: _t->setMinorTickLabelFullNumber((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->setMinTicks((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 32: _t->setMinorTicks((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 33: _t->setMinorTicks((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 34: _t->setTickOutsideLength((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 35: _t->setMinorTickOutsideLength((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 36: _t->setTickInsideLength((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 37: _t->setMinorTickInsideLength((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 38: _t->setAxisColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 39: _t->setShowZeroAxis((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->setInverted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->setGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 42: _t->setGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 43: _t->setMinorGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 44: _t->setMinorGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 45: _t->setGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 46: _t->setGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 47: _t->setMinorGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 48: _t->setMinorGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 49: _t->setDrawMode1((*reinterpret_cast<std::add_pointer_t<JKQTPCADrawMode>>(_a[1]))); break;
        case 50: _t->setDrawMode0((*reinterpret_cast<std::add_pointer_t<JKQTPCADrawMode>>(_a[1]))); break;
        case 51: _t->setDrawMode2((*reinterpret_cast<std::add_pointer_t<JKQTPCADrawMode>>(_a[1]))); break;
        case 52: _t->setMinorTickWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 53: _t->setTickWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 54: _t->setLineWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 55: _t->setLineWidthZeroAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 56: _t->setTickLabelDistance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 57: _t->setLabelDistance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 58: _t->setLabelDigits((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->setDrawGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 60: _t->setDrawMinorGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: _t->setTickLabelAngle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 62: _t->setArrowSizeFactor((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 63: _t->setLabelColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 64: _t->setMinorTickColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 65: _t->setMinorTickLabelColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 66: _t->setTickColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 67: _t->setTickLabelColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 68: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPCoordinateAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPCoordinateAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPCoordinateAxisE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JKQTPCoordinateAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 69)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 69;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 69)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 69;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPVerticalAxisBase::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPVerticalAxisBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPVerticalAxisBase, qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPVerticalAxisBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPCoordinateAxis::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPVerticalAxisBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPVerticalAxisBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPVerticalAxisBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPVerticalAxisBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVerticalAxisBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPCoordinateAxis::qt_metacast(_clname);
}

int JKQTPVerticalAxisBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPCoordinateAxis::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17JKQTPVerticalAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPVerticalAxis::qt_create_metaobjectdata<qt_meta_tag_ZN17JKQTPVerticalAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPVerticalAxis"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPVerticalAxis, qt_meta_tag_ZN17JKQTPVerticalAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPVerticalAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPVerticalAxisBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPVerticalAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPVerticalAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17JKQTPVerticalAxisE_t>.metaTypes,
    nullptr
} };

void JKQTPVerticalAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPVerticalAxis *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPVerticalAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPVerticalAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPVerticalAxisE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPVerticalAxisBase::qt_metacast(_clname);
}

int JKQTPVerticalAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPVerticalAxisBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPVerticalIndependentAxis::qt_create_metaobjectdata<qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPVerticalIndependentAxis",
        "setAxisOffset",
        "",
        "__value",
        "setAxisWidth",
        "setOtherAxisOffset",
        "setOtherAxisWidth",
        "setOtherAxisInverted"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setAxisOffset'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAxisWidth'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisOffset'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisWidth'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisInverted'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPVerticalIndependentAxis, qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPVerticalIndependentAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPVerticalAxisBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>.metaTypes,
    nullptr
} };

void JKQTPVerticalIndependentAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPVerticalIndependentAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setAxisOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->setAxisWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setOtherAxisOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setOtherAxisWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setOtherAxisInverted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPVerticalIndependentAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPVerticalIndependentAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPVerticalIndependentAxisE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPVerticalAxisBase::qt_metacast(_clname);
}

int JKQTPVerticalIndependentAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPVerticalAxisBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPHorizontalAxisBase::qt_create_metaobjectdata<qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPHorizontalAxisBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPHorizontalAxisBase, qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPHorizontalAxisBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPCoordinateAxis::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPHorizontalAxisBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPHorizontalAxisBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPHorizontalAxisBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPHorizontalAxisBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPHorizontalAxisBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPCoordinateAxis::qt_metacast(_clname);
}

int JKQTPHorizontalAxisBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPCoordinateAxis::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN19JKQTPHorizontalAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPHorizontalAxis::qt_create_metaobjectdata<qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPHorizontalAxis"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPHorizontalAxis, qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPHorizontalAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPHorizontalAxisBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>.metaTypes,
    nullptr
} };

void JKQTPHorizontalAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPHorizontalAxis *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPHorizontalAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPHorizontalAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPHorizontalAxisE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPHorizontalAxisBase::qt_metacast(_clname);
}

int JKQTPHorizontalAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPHorizontalAxisBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPHorizontalIndependentAxis::qt_create_metaobjectdata<qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPHorizontalIndependentAxis",
        "setAxisOffset",
        "",
        "__value",
        "setAxisWidth",
        "setOtherAxisOffset",
        "setOtherAxisWidth",
        "setOtherAxisInverted"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setAxisOffset'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAxisWidth'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisOffset'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisWidth'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setOtherAxisInverted'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPHorizontalIndependentAxis, qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPHorizontalIndependentAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPHorizontalAxis::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>.metaTypes,
    nullptr
} };

void JKQTPHorizontalIndependentAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPHorizontalIndependentAxis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setAxisOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->setAxisWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setOtherAxisOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setOtherAxisWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setOtherAxisInverted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPHorizontalIndependentAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPHorizontalIndependentAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPHorizontalIndependentAxisE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPHorizontalAxis::qt_metacast(_clname);
}

int JKQTPHorizontalIndependentAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPHorizontalAxis::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
