/****************************************************************************
** Meta object code from reading C++ file 'jkqtpkey.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpkey.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpkey.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12JKQTPBaseKeyE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBaseKey::qt_create_metaobjectdata<qt_meta_tag_ZN12JKQTPBaseKeyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBaseKey",
        "setVisible",
        "",
        "__value",
        "setFrameVisible",
        "setFrameWidth",
        "setFrameRounding",
        "setFontSize",
        "setSampleLineLength",
        "setXMargin",
        "setYMargin",
        "setXOffset",
        "setYOffset",
        "setXSeparation",
        "setYSeparation",
        "setColumnSeparation",
        "setPosition",
        "JKQTPKeyPosition",
        "setLayout",
        "JKQTPKeyLayout",
        "setFrameColor",
        "QColor",
        "setFrameLineStyle",
        "Qt::PenStyle",
        "setTextColor",
        "setBackgroundColor",
        "Qt::BrushStyle",
        "__style",
        "setBackgroundBrush",
        "QBrush"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setVisible'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFrameVisible'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFrameWidth'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setFrameRounding'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setSampleLineLength'
        QtMocHelpers::SlotData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setXMargin'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setYMargin'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setXOffset'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setYOffset'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setXSeparation'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setYSeparation'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setColumnSeparation'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setPosition'
        QtMocHelpers::SlotData<void(JKQTPKeyPosition)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 3 },
        }}),
        // Slot 'setLayout'
        QtMocHelpers::SlotData<void(JKQTPKeyLayout)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 3 },
        }}),
        // Slot 'setFrameColor'
        QtMocHelpers::SlotData<void(QColor)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 3 },
        }}),
        // Slot 'setFrameLineStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 3 },
        }}),
        // Slot 'setTextColor'
        QtMocHelpers::SlotData<void(QColor)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 3 },
        }}),
        // Slot 'setBackgroundColor'
        QtMocHelpers::SlotData<void(QColor)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 3 },
        }}),
        // Slot 'setBackgroundColor'
        QtMocHelpers::SlotData<void(QColor, Qt::BrushStyle)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 3 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'setBackgroundBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBaseKey, qt_meta_tag_ZN12JKQTPBaseKeyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBaseKey::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPBaseKeyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPBaseKeyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12JKQTPBaseKeyE_t>.metaTypes,
    nullptr
} };

void JKQTPBaseKey::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBaseKey *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setFrameVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setFrameWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setFrameRounding((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->setSampleLineLength((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->setXMargin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setYMargin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setXOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->setYOffset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->setXSeparation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->setYSeparation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->setColumnSeparation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->setPosition((*reinterpret_cast<std::add_pointer_t<JKQTPKeyPosition>>(_a[1]))); break;
        case 14: _t->setLayout((*reinterpret_cast<std::add_pointer_t<JKQTPKeyLayout>>(_a[1]))); break;
        case 15: _t->setFrameColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 16: _t->setFrameLineStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 17: _t->setTextColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 18: _t->setBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 19: _t->setBackgroundColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::BrushStyle>>(_a[2]))); break;
        case 20: _t->setBackgroundBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBaseKey::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBaseKey::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPBaseKeyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JKQTPBaseKey::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN12JKQTPMainKeyE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPMainKey::qt_create_metaobjectdata<qt_meta_tag_ZN12JKQTPMainKeyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPMainKey"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPMainKey, qt_meta_tag_ZN12JKQTPMainKeyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPMainKey::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBaseKey::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPMainKeyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPMainKeyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12JKQTPMainKeyE_t>.metaTypes,
    nullptr
} };

void JKQTPMainKey::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPMainKey *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPMainKey::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPMainKey::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPMainKeyE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPBaseKey::qt_metacast(_clname);
}

int JKQTPMainKey::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBaseKey::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
