/****************************************************************************
** Meta object code from reading C++ file 'jkqtpbarchart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpbarchart.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpbarchart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarVerticalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarVerticalGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarVerticalGraph, qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarVerticalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarVerticalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarVerticalGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPBarVerticalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarVerticalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBarVerticalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPBarGraphBase::qt_metacast(_clname);
}

int JKQTPBarVerticalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarVerticalErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarVerticalErrorGraph",
        "setBarErrorSymmetric",
        "",
        "__value",
        "setBarErrorStyle",
        "JKQTPErrorPlotstyle",
        "setBarErrorColumn",
        "column",
        "size_t",
        "setBarLowerErrorColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setBarErrorSymmetric'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setBarErrorStyle'
        QtMocHelpers::SlotData<void(JKQTPErrorPlotstyle)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
        // Slot 'setBarErrorColumn'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setBarErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setBarLowerErrorColumn'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setBarLowerErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarVerticalErrorGraph, qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarVerticalErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarVerticalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarVerticalErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarVerticalErrorGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setBarErrorSymmetric((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setBarErrorStyle((*reinterpret_cast<std::add_pointer_t<JKQTPErrorPlotstyle>>(_a[1]))); break;
        case 2: _t->setBarErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setBarErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setBarLowerErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setBarLowerErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBarVerticalErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarVerticalErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPBarVerticalErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPYGraphErrors"))
        return static_cast< JKQTPYGraphErrors*>(this);
    return JKQTPBarVerticalGraph::qt_metacast(_clname);
}

int JKQTPBarVerticalErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarVerticalGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarHorizontalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarHorizontalGraph",
        "setKeyColumn",
        "",
        "__value",
        "setValueColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setValueColumn'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarHorizontalGraph, qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarHorizontalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarHorizontalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarHorizontalGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setValueColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBarHorizontalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarHorizontalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBarHorizontalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPBarGraphBase::qt_metacast(_clname);
}

int JKQTPBarHorizontalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarHorizontalErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarHorizontalErrorGraph",
        "setBarErrorSymmetric",
        "",
        "__value",
        "setBarErrorStyle",
        "JKQTPErrorPlotstyle",
        "setBarErrorColumn",
        "column",
        "size_t",
        "setBarLowerErrorColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setBarErrorSymmetric'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setBarErrorStyle'
        QtMocHelpers::SlotData<void(JKQTPErrorPlotstyle)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
        // Slot 'setBarErrorColumn'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setBarErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setBarLowerErrorColumn'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setBarLowerErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarHorizontalErrorGraph, qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarHorizontalErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarHorizontalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarHorizontalErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarHorizontalErrorGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setBarErrorSymmetric((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setBarErrorStyle((*reinterpret_cast<std::add_pointer_t<JKQTPErrorPlotstyle>>(_a[1]))); break;
        case 2: _t->setBarErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setBarErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setBarLowerErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setBarLowerErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBarHorizontalErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarHorizontalErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPBarHorizontalErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPXGraphErrors"))
        return static_cast< JKQTPXGraphErrors*>(this);
    return JKQTPBarHorizontalGraph::qt_metacast(_clname);
}

int JKQTPBarHorizontalErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarHorizontalGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarVerticalStackableGraph::qt_create_metaobjectdata<qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarVerticalStackableGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarVerticalStackableGraph, qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarVerticalStackableGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarVerticalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarVerticalStackableGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarVerticalStackableGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPBarVerticalStackableGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarVerticalStackableGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPBarVerticalStackableGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPBarGraphStackInternalInterface"))
        return static_cast< JKQTPBarGraphStackInternalInterface*>(this);
    return JKQTPBarVerticalGraph::qt_metacast(_clname);
}

int JKQTPBarVerticalStackableGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarVerticalGraph::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarHorizontalStackableGraph::qt_create_metaobjectdata<qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarHorizontalStackableGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBarHorizontalStackableGraph, qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarHorizontalStackableGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPBarHorizontalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPBarHorizontalStackableGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarHorizontalStackableGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPBarHorizontalStackableGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarHorizontalStackableGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPBarHorizontalStackableGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPBarGraphStackInternalInterface"))
        return static_cast< JKQTPBarGraphStackInternalInterface*>(this);
    return JKQTPBarHorizontalGraph::qt_metacast(_clname);
}

int JKQTPBarHorizontalStackableGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPBarHorizontalGraph::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
