/****************************************************************************
** Meta object code from reading C++ file 'jkqtpvectorfield.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpvectorfield.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpvectorfield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPVectorFieldGraph::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPVectorFieldGraph",
        "vectorLengthMode",
        "VectorLengthMode",
        "autoscaleLengthFactor",
        "lengthScaleFactor",
        "anchorPoint",
        "VectorAnchorPoint",
        "minLineWidth",
        "vectorLineWidthMode",
        "VectorLineWidthMode",
        "AnchorBottom",
        "AnchorMid",
        "AnchorTip",
        "AutoscaleLength",
        "LengthFromData",
        "IgnoreLength",
        "DefaultVectorLineWidth",
        "AutoscaleLineWidthFromLength"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'vectorLengthMode'
        QtMocHelpers::PropertyData<enum VectorLengthMode>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'autoscaleLengthFactor'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'lengthScaleFactor'
        QtMocHelpers::PropertyData<double>(4, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'anchorPoint'
        QtMocHelpers::PropertyData<enum VectorAnchorPoint>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'minLineWidth'
        QtMocHelpers::PropertyData<double>(7, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'vectorLineWidthMode'
        QtMocHelpers::PropertyData<enum VectorLineWidthMode>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VectorAnchorPoint'
        QtMocHelpers::EnumData<enum VectorAnchorPoint>(6, 6, QMC::EnumFlags{}).add({
            {   10, VectorAnchorPoint::AnchorBottom },
            {   11, VectorAnchorPoint::AnchorMid },
            {   12, VectorAnchorPoint::AnchorTip },
        }),
        // enum 'VectorLengthMode'
        QtMocHelpers::EnumData<enum VectorLengthMode>(2, 2, QMC::EnumFlags{}).add({
            {   13, VectorLengthMode::AutoscaleLength },
            {   14, VectorLengthMode::LengthFromData },
            {   15, VectorLengthMode::IgnoreLength },
        }),
        // enum 'VectorLineWidthMode'
        QtMocHelpers::EnumData<enum VectorLineWidthMode>(9, 9, QMC::EnumFlags{}).add({
            {   16, VectorLineWidthMode::DefaultVectorLineWidth },
            {   17, VectorLineWidthMode::AutoscaleLineWidthFromLength },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPVectorFieldGraph, qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPVectorFieldGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYAndVectorGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPVectorFieldGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPVectorFieldGraph *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum VectorLengthMode*>(_v) = _t->getVectorLengthMode(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getAutoscaleLengthFactor(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getLengthScaleFactor(); break;
        case 3: *reinterpret_cast<enum VectorAnchorPoint*>(_v) = _t->getAnchorPoint(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getMinLineWIdth(); break;
        case 5: *reinterpret_cast<enum VectorLineWidthMode*>(_v) = _t->getVectorLineWidthMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVectorLengthMode(*reinterpret_cast<enum VectorLengthMode*>(_v)); break;
        case 1: _t->setAutoscaleLengthFactor(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLengthScaleFactor(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setAnchorPoint(*reinterpret_cast<enum VectorAnchorPoint*>(_v)); break;
        case 4: _t->setMinLineWidth(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setVectorLineWidthMode(*reinterpret_cast<enum VectorLineWidthMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPVectorFieldGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPVectorFieldGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPVectorFieldGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphDecoratedLineStyleMixin"))
        return static_cast< JKQTPGraphDecoratedLineStyleMixin*>(this);
    return JKQTPXYAndVectorGraph::qt_metacast(_clname);
}

int JKQTPVectorFieldGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYAndVectorGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPParametrizedVectorFieldGraph::qt_create_metaobjectdata<qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPParametrizedVectorFieldGraph",
        "VectorColorMode",
        "DefaultColor",
        "ColorFromMagnitude",
        "ColorFromAngle",
        "ColorFromCustomColumn"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VectorColorMode'
        QtMocHelpers::EnumData<enum VectorColorMode>(1, 1, QMC::EnumFlags{}).add({
            {    2, VectorColorMode::DefaultColor },
            {    3, VectorColorMode::ColorFromMagnitude },
            {    4, VectorColorMode::ColorFromAngle },
            {    5, VectorColorMode::ColorFromCustomColumn },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPParametrizedVectorFieldGraph, qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPParametrizedVectorFieldGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPVectorFieldGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPParametrizedVectorFieldGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPParametrizedVectorFieldGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPParametrizedVectorFieldGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPParametrizedVectorFieldGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPParametrizedVectorFieldGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPColorPaletteStyleAndToolsMixin"))
        return static_cast< JKQTPColorPaletteStyleAndToolsMixin*>(this);
    return JKQTPVectorFieldGraph::qt_metacast(_clname);
}

int JKQTPParametrizedVectorFieldGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPVectorFieldGraph::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
