/****************************************************************************
** Meta object code from reading C++ file 'jkqtpimage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpimage.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpimage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14JKQTPImageBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImageBase::qt_create_metaobjectdata<qt_meta_tag_ZN14JKQTPImageBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImageBase",
        "setX",
        "",
        "__value",
        "setY",
        "setWidth",
        "setHeight"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setX'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setWidth'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setHeight'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImageBase, qt_meta_tag_ZN14JKQTPImageBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImageBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPImageBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPImageBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14JKQTPImageBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPImageBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImageBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->setY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setHeight((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImageBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImageBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPImageBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPImageBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN18JKQTPMathImageBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPMathImageBase::qt_create_metaobjectdata<qt_meta_tag_ZN18JKQTPMathImageBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPMathImageBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPMathImageBase, qt_meta_tag_ZN18JKQTPMathImageBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPMathImageBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImageBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPMathImageBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPMathImageBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18JKQTPMathImageBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPMathImageBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPMathImageBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPMathImageBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPMathImageBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPMathImageBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPImageBase::qt_metacast(_clname);
}

int JKQTPMathImageBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImageBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10JKQTPImageE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImage::qt_create_metaobjectdata<qt_meta_tag_ZN10JKQTPImageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImage",
        "saveImagePlotAsImage",
        "",
        "filename",
        "outputFormat",
        "copyImagePlotAsImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QByteArray, 4 },
        }}),
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copyImagePlotAsImage'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImage, qt_meta_tag_ZN10JKQTPImageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImage::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImageBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPImageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPImageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10JKQTPImageE_t>.metaTypes,
    nullptr
} };

void JKQTPImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->saveImagePlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 1: _t->saveImagePlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->saveImagePlotAsImage(); break;
        case 3: _t->copyImagePlotAsImage(); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPImageE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPImageBase::qt_metacast(_clname);
}

int JKQTPImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImageBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN14JKQTPMathImageE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPMathImage::qt_create_metaobjectdata<qt_meta_tag_ZN14JKQTPMathImageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPMathImage",
        "saveImagePlotAsImage",
        "",
        "filename",
        "outputFormat",
        "copyImagePlotAsImage",
        "saveColorbarPlotAsImage",
        "copyColorbarPlotAsImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QByteArray, 4 },
        }}),
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'saveImagePlotAsImage'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copyImagePlotAsImage'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveColorbarPlotAsImage'
        QtMocHelpers::SlotData<void(const QString &, const QByteArray &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QByteArray, 4 },
        }}),
        // Slot 'saveColorbarPlotAsImage'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'saveColorbarPlotAsImage'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copyColorbarPlotAsImage'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPMathImage, qt_meta_tag_ZN14JKQTPMathImageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPMathImage::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPMathImageBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPMathImageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPMathImageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14JKQTPMathImageE_t>.metaTypes,
    nullptr
} };

void JKQTPMathImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPMathImage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->saveImagePlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 1: _t->saveImagePlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->saveImagePlotAsImage(); break;
        case 3: _t->copyImagePlotAsImage(); break;
        case 4: _t->saveColorbarPlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 5: _t->saveColorbarPlotAsImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->saveColorbarPlotAsImage(); break;
        case 7: _t->copyColorbarPlotAsImage(); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPMathImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPMathImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPMathImageE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPColorPaletteWithModifierStyleAndToolsMixin"))
        return static_cast< JKQTPColorPaletteWithModifierStyleAndToolsMixin*>(this);
    return JKQTPMathImageBase::qt_metacast(_clname);
}

int JKQTPMathImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPMathImageBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN20JKQTPColumnMathImageE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPColumnMathImage::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPColumnMathImageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPColumnMathImage"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPColumnMathImage, qt_meta_tag_ZN20JKQTPColumnMathImageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPColumnMathImage::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPMathImage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPColumnMathImageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPColumnMathImageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPColumnMathImageE_t>.metaTypes,
    nullptr
} };

void JKQTPColumnMathImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPColumnMathImage *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPColumnMathImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPColumnMathImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPColumnMathImageE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPMathImage::qt_metacast(_clname);
}

int JKQTPColumnMathImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPMathImage::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
