/****************************************************************************
** Meta object code from reading C++ file 'jkqtpboxplotbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpboxplotbase.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpboxplotbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBoxplotGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBoxplotGraphBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setDataSortOrder",
        "DataSortOrder",
        "__value",
        "setPositionColumn",
        "size_t",
        "setMedianColumn",
        "setMeanColumn",
        "setMinColumn",
        "setMaxColumn",
        "setPercentile25Column",
        "setPercentile75Column",
        "setMedianConfidenceColumn",
        "setBoxWidthRelative",
        "setUseRelativeBoxWidth"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(enum DataSortOrder)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setPositionColumn'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setPositionColumn'
        QtMocHelpers::SlotData<void(size_t)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setMedianColumn'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setMedianColumn'
        QtMocHelpers::SlotData<void(size_t)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setMeanColumn'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setMeanColumn'
        QtMocHelpers::SlotData<void(size_t)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setMinColumn'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setMinColumn'
        QtMocHelpers::SlotData<void(size_t)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setMaxColumn'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setMaxColumn'
        QtMocHelpers::SlotData<void(size_t)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setPercentile25Column'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setPercentile25Column'
        QtMocHelpers::SlotData<void(size_t)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setPercentile75Column'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setPercentile75Column'
        QtMocHelpers::SlotData<void(size_t)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setMedianConfidenceColumn'
        QtMocHelpers::SlotData<void(size_t)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setBoxWidthRelative'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setUseRelativeBoxWidth'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBoxplotGraphBase, qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBoxplotGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPBoxplotGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBoxplotGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<enum DataSortOrder>>(_a[1]))); break;
        case 2: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setPositionColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setPositionColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 5: _t->setMedianColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->setMedianColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 7: _t->setMeanColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->setMeanColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 9: _t->setMinColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->setMinColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 11: _t->setMaxColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setMaxColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 13: _t->setPercentile25Column((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->setPercentile25Column((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 15: _t->setPercentile75Column((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->setPercentile75Column((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 17: _t->setMedianConfidenceColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 18: _t->setBoxWidthRelative((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->setUseRelativeBoxWidth((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBoxplotGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBoxplotGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPBoxplotGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphBoxplotStyleMixin"))
        return static_cast< JKQTPGraphBoxplotStyleMixin*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPBoxplotGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBoxplotElementBase::qt_create_metaobjectdata<qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBoxplotElementBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setPos",
        "__value",
        "setMedian",
        "setMin",
        "setMean",
        "setMax",
        "setPercentile25",
        "setPercentile75",
        "setData",
        "pos",
        "min",
        "q25",
        "median",
        "mean",
        "q75",
        "max",
        "setDrawMedian",
        "setDrawMinMax",
        "setDrawNotch",
        "setMedianConfidenceIntervalWidth"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setPos'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMedian'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMin'
        QtMocHelpers::SlotData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMean'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMax'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setPercentile25'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setPercentile75'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setData'
        QtMocHelpers::SlotData<void(double, double, double, double, double, double, double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 14 }, { QMetaType::Double, 15 }, { QMetaType::Double, 16 }, { QMetaType::Double, 17 },
            { QMetaType::Double, 18 }, { QMetaType::Double, 19 }, { QMetaType::Double, 20 },
        }}),
        // Slot 'setDrawMedian'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawMinMax'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawNotch'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setMedianConfidenceIntervalWidth'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPBoxplotElementBase, qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBoxplotElementBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPBoxplotElementBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBoxplotElementBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setPos((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setMedian((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setMean((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->setMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->setPercentile25((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setPercentile75((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setData((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7]))); break;
        case 9: _t->setDrawMedian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setDrawMinMax((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setDrawNotch((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setMedianConfidenceIntervalWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBoxplotElementBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBoxplotElementBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPBoxplotElementBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphBoxplotStyleMixin"))
        return static_cast< JKQTPGraphBoxplotStyleMixin*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPBoxplotElementBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
