/****************************************************************************
** Meta object code from reading C++ file 'jkqtpevaluatedfunctionbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpevaluatedfunctionbase.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpevaluatedfunctionbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPEvaluatedFunctionGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPEvaluatedFunctionGraphBase",
        "setMinSamples",
        "",
        "__value",
        "setMaxRefinementDegree",
        "setSlopeTolerance",
        "setMinPixelPerSample",
        "setDataCleanupMaxAllowedAngleDegree",
        "setDisplaySamplePoints",
        "setParamsV",
        "p1",
        "p2",
        "p3",
        "p4",
        "p5",
        "setParameterColumn",
        "size_t"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setMinSamples'
        QtMocHelpers::SlotData<void(const unsigned int &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'setMaxRefinementDegree'
        QtMocHelpers::SlotData<void(const unsigned int &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 3 },
        }}),
        // Slot 'setSlopeTolerance'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setMinPixelPerSample'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDataCleanupMaxAllowedAngleDegree'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDisplaySamplePoints'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setParamsV'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'setParamsV'
        QtMocHelpers::SlotData<void(double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'setParamsV'
        QtMocHelpers::SlotData<void(double, double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 },
        }}),
        // Slot 'setParamsV'
        QtMocHelpers::SlotData<void(double, double, double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'setParamsV'
        QtMocHelpers::SlotData<void(double, double, double, double, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
            { QMetaType::Double, 14 },
        }}),
        // Slot 'setParameterColumn'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setParameterColumn'
        QtMocHelpers::SlotData<void(size_t)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPEvaluatedFunctionGraphBase, qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPEvaluatedFunctionGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPEvaluatedFunctionGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPEvaluatedFunctionGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setMinSamples((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 1: _t->setMaxRefinementDegree((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 2: _t->setSlopeTolerance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->setMinPixelPerSample((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setDataCleanupMaxAllowedAngleDegree((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->setDisplaySamplePoints((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 8: _t->setParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 9: _t->setParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 10: _t->setParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5]))); break;
        case 11: _t->setParameterColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setParameterColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPEvaluatedFunctionGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPEvaluatedFunctionGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPEvaluatedFunctionGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPEvaluatedFunctionGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPEvaluatedFunctionWithErrorsGraphBase",
        "setErrorParams",
        "",
        "QList<double>",
        "errorParams",
        "setErrorParameterColumn",
        "__value",
        "size_t",
        "setErrorParamsV",
        "p1",
        "p2",
        "p3",
        "p4",
        "p5"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setErrorParams'
        QtMocHelpers::SlotData<void(const QVector<double> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setErrorParameterColumn'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setErrorParameterColumn'
        QtMocHelpers::SlotData<void(size_t)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 6 },
        }}),
        // Slot 'setErrorParamsV'
        QtMocHelpers::SlotData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'setErrorParamsV'
        QtMocHelpers::SlotData<void(double, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
        }}),
        // Slot 'setErrorParamsV'
        QtMocHelpers::SlotData<void(double, double, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 }, { QMetaType::Double, 10 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'setErrorParamsV'
        QtMocHelpers::SlotData<void(double, double, double, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 }, { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 },
        }}),
        // Slot 'setErrorParamsV'
        QtMocHelpers::SlotData<void(double, double, double, double, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 }, { QMetaType::Double, 10 }, { QMetaType::Double, 11 }, { QMetaType::Double, 12 },
            { QMetaType::Double, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPEvaluatedFunctionWithErrorsGraphBase, qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPEvaluatedFunctionWithErrorsGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPEvaluatedFunctionGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPEvaluatedFunctionWithErrorsGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setErrorParams((*reinterpret_cast<std::add_pointer_t<QList<double>>>(_a[1]))); break;
        case 1: _t->setErrorParameterColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setErrorParameterColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 3: _t->setErrorParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setErrorParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: _t->setErrorParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 6: _t->setErrorParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 7: _t->setErrorParamsV((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<double> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *JKQTPEvaluatedFunctionWithErrorsGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41JKQTPEvaluatedFunctionWithErrorsGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPEvaluatedFunctionGraphBase::qt_metacast(_clname);
}

int JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPEvaluatedFunctionGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
