/****************************************************************************
** Meta object code from reading C++ file 'jkqtpgraphsbaseerrors.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpgraphsbaseerrors.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpgraphsbaseerrors.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGraphErrorStyleMixin::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGraphErrorStyleMixin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGraphErrorStyleMixin, qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGraphErrorStyleMixin::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTPGraphErrorStyleMixinE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXGraphErrorData::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXGraphErrorData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXGraphErrorData, qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXGraphErrorData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPXGraphErrorDataE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPYGraphErrorData::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPYGraphErrorData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPYGraphErrorData, qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPYGraphErrorData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPYGraphErrorDataE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17JKQTPXGraphErrorsE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXGraphErrors::qt_create_metaobjectdata<qt_meta_tag_ZN17JKQTPXGraphErrorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXGraphErrors"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXGraphErrors, qt_meta_tag_ZN17JKQTPXGraphErrorsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXGraphErrors::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTPXGraphErrorData>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPXGraphErrorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPXGraphErrorsE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17JKQTPXGraphErrorsE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17JKQTPYGraphErrorsE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPYGraphErrors::qt_create_metaobjectdata<qt_meta_tag_ZN17JKQTPYGraphErrorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPYGraphErrors"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPYGraphErrors, qt_meta_tag_ZN17JKQTPYGraphErrorsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPYGraphErrors::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTPYGraphErrorData>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPYGraphErrorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPYGraphErrorsE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17JKQTPYGraphErrorsE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYGraphErrors::qt_create_metaobjectdata<qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYGraphErrors"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYGraphErrors, qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYGraphErrors::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTPXGraphErrorData>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18JKQTPXYGraphErrorsE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
