/****************************************************************************
** Meta object code from reading C++ file 'jkqtpenhancedtableview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/gui/jkqtpenhancedtableview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpenhancedtableview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPEnhancedTableView::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPEnhancedTableView",
        "keyPressed",
        "",
        "key",
        "Qt::KeyboardModifiers",
        "modifiers",
        "text",
        "copySelectionToExcel",
        "copyrole",
        "storeHead",
        "copySelectionToExcelNoHead",
        "copySelectionToCSV",
        "separator",
        "decimalpoint",
        "copySelectionToCSVNoHead",
        "print"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyPressed'
        QtMocHelpers::SignalData<void(int, Qt::KeyboardModifiers, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'copySelectionToExcel'
        QtMocHelpers::SlotData<void(int, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'copySelectionToExcel'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'copySelectionToExcel'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copySelectionToExcelNoHead'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'copySelectionToExcelNoHead'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copySelectionToCSV'
        QtMocHelpers::SlotData<void(int, bool, const QString &, const QChar &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 }, { QMetaType::QString, 12 }, { QMetaType::QChar, 13 },
        }}),
        // Slot 'copySelectionToCSV'
        QtMocHelpers::SlotData<void(int, bool, const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'copySelectionToCSV'
        QtMocHelpers::SlotData<void(int, bool)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'copySelectionToCSV'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'copySelectionToCSV'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copySelectionToCSVNoHead'
        QtMocHelpers::SlotData<void(int, const QString &, const QChar &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 12 }, { QMetaType::QChar, 13 },
        }}),
        // Slot 'copySelectionToCSVNoHead'
        QtMocHelpers::SlotData<void(int, const QString &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'copySelectionToCSVNoHead'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'copySelectionToCSVNoHead'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPEnhancedTableView, qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPEnhancedTableView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTableView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>.metaTypes,
    nullptr
} };

void JKQTPEnhancedTableView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPEnhancedTableView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->copySelectionToExcel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->copySelectionToExcel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->copySelectionToExcel(); break;
        case 4: _t->copySelectionToExcelNoHead((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->copySelectionToExcelNoHead(); break;
        case 6: _t->copySelectionToCSV((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QChar>>(_a[4]))); break;
        case 7: _t->copySelectionToCSV((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 8: _t->copySelectionToCSV((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->copySelectionToCSV((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->copySelectionToCSV(); break;
        case 11: _t->copySelectionToCSVNoHead((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QChar>>(_a[3]))); break;
        case 12: _t->copySelectionToCSVNoHead((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 13: _t->copySelectionToCSVNoHead((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->copySelectionToCSVNoHead(); break;
        case 15: _t->print(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JKQTPEnhancedTableView::*)(int , Qt::KeyboardModifiers , const QString & )>(_a, &JKQTPEnhancedTableView::keyPressed, 0))
            return;
    }
}

const QMetaObject *JKQTPEnhancedTableView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPEnhancedTableView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPEnhancedTableViewE_t>.strings))
        return static_cast<void*>(this);
    return QTableView::qt_metacast(_clname);
}

int JKQTPEnhancedTableView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void JKQTPEnhancedTableView::keyPressed(int _t1, Qt::KeyboardModifiers _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
