/****************************************************************************
** Meta object code from reading C++ file 'jkqtpfilledcurve.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpfilledcurve.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpfilledcurve.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledCurveGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledCurveGraphBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setFillMode",
        "JKQTPFilledCurveGraphBase::FillMode",
        "mode",
        "FillMode",
        "SingleFilling",
        "TwoColorFilling"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setFillMode'
        QtMocHelpers::SlotData<void(JKQTPFilledCurveGraphBase::FillMode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FillMode'
        QtMocHelpers::EnumData<enum FillMode>(8, 8, QMC::EnumFlags{}).add({
            {    9, FillMode::SingleFilling },
            {   10, FillMode::TwoColorFilling },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledCurveGraphBase, qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledCurveGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYBaselineGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledCurveGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledCurveGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setFillMode((*reinterpret_cast<std::add_pointer_t<JKQTPFilledCurveGraphBase::FillMode>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPFilledCurveGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledCurveGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFilledCurveGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineAndFillStyleMixin"))
        return static_cast< JKQTPGraphLineAndFillStyleMixin*>(this);
    return JKQTPXYBaselineGraph::qt_metacast(_clname);
}

int JKQTPFilledCurveGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYBaselineGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledCurveXGraph::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledCurveXGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledCurveXGraph, qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledCurveXGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFilledCurveGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledCurveXGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledCurveXGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFilledCurveXGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledCurveXGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveXGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPFilledCurveGraphBase::qt_metacast(_clname);
}

int JKQTPFilledCurveXGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFilledCurveGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledCurveXErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledCurveXErrorGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledCurveXErrorGraph, qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledCurveXErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFilledCurveXGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledCurveXErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledCurveXErrorGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFilledCurveXErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledCurveXErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveXErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPYGraphErrors"))
        return static_cast< JKQTPYGraphErrors*>(this);
    return JKQTPFilledCurveXGraph::qt_metacast(_clname);
}

int JKQTPFilledCurveXErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFilledCurveXGraph::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledCurveYGraph::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledCurveYGraph",
        "setKeyColumn",
        "",
        "__value",
        "setValueColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setValueColumn'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledCurveYGraph, qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledCurveYGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFilledCurveGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledCurveYGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledCurveYGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setValueColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPFilledCurveYGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledCurveYGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPFilledCurveYGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPFilledCurveGraphBase::qt_metacast(_clname);
}

int JKQTPFilledCurveYGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFilledCurveGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledCurveYErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledCurveYErrorGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledCurveYErrorGraph, qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledCurveYErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFilledCurveYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledCurveYErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledCurveYErrorGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFilledCurveYErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledCurveYErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPFilledCurveYErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPXGraphErrors"))
        return static_cast< JKQTPXGraphErrors*>(this);
    return JKQTPFilledCurveYGraph::qt_metacast(_clname);
}

int JKQTPFilledCurveYErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFilledCurveYGraph::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledVerticalRangeGraph::qt_create_metaobjectdata<qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledVerticalRangeGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledVerticalRangeGraph, qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledVerticalRangeGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledVerticalRangeGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledVerticalRangeGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFilledVerticalRangeGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledVerticalRangeGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPFilledVerticalRangeGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineAndFillStyleMixin"))
        return static_cast< JKQTPGraphLineAndFillStyleMixin*>(this);
    return JKQTPXYYGraph::qt_metacast(_clname);
}

int JKQTPFilledVerticalRangeGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYYGraph::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFilledHorizontalRangeGraph::qt_create_metaobjectdata<qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFilledHorizontalRangeGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFilledHorizontalRangeGraph, qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFilledHorizontalRangeGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXXYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFilledHorizontalRangeGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFilledHorizontalRangeGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFilledHorizontalRangeGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFilledHorizontalRangeGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPFilledHorizontalRangeGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineAndFillStyleMixin"))
        return static_cast< JKQTPGraphLineAndFillStyleMixin*>(this);
    return JKQTPXXYGraph::qt_metacast(_clname);
}

int JKQTPFilledHorizontalRangeGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXXYGraph::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
