/****************************************************************************
** Meta object code from reading C++ file 'jkqtpgraphlabelstylemixin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpgraphlabelstylemixin.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpgraphlabelstylemixin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGraphValueLabelStyleMixin::qt_create_metaobjectdata<qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGraphValueLabelStyleMixin",
        "labelOffset",
        "labelOffsetWithConnector",
        "labelBoxRounding",
        "labelXPadding",
        "labelYPadding",
        "labelPosition",
        "JKQTPGraphLabelPosition",
        "drawLabelBoxFrame",
        "labelBoxType",
        "JKQTPGraphLabelBoxType"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'labelOffset'
        QtMocHelpers::PropertyData<double>(1, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelOffsetWithConnector'
        QtMocHelpers::PropertyData<double>(2, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelBoxRounding'
        QtMocHelpers::PropertyData<double>(3, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelXPadding'
        QtMocHelpers::PropertyData<double>(4, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelYPadding'
        QtMocHelpers::PropertyData<double>(5, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelPosition'
        QtMocHelpers::PropertyData<JKQTPGraphLabelPosition>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'drawLabelBoxFrame'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'labelBoxType'
        QtMocHelpers::PropertyData<JKQTPGraphLabelBoxType>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGraphValueLabelStyleMixin, qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGraphValueLabelStyleMixin::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTPGraphTextStyleMixin>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30JKQTPGraphValueLabelStyleMixinE_t>.metaTypes,
    nullptr
} };

void JKQTPGraphValueLabelStyleMixin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<JKQTPGraphValueLabelStyleMixin *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getLabelOffset(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getLabelOffsetWithConnector(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getLabelBoxRounding(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getLabelXPadding(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getLabelYPadding(); break;
        case 5: *reinterpret_cast<JKQTPGraphLabelPosition*>(_v) = _t->getLabelPosition(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->drawsLabelBoxFrame(); break;
        case 7: *reinterpret_cast<JKQTPGraphLabelBoxType*>(_v) = _t->getLabelBoxType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLabelOffset(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setLabelOffsetWithConnector(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setLabelBoxRounding(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setLabelXPadding(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setLabelYPadding(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setLabelPosition(*reinterpret_cast<JKQTPGraphLabelPosition*>(_v)); break;
        case 6: _t->setDrawLabelBoxFrame(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setLabelBoxType(*reinterpret_cast<JKQTPGraphLabelBoxType*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYLabelsGeneratorMixin::qt_create_metaobjectdata<qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYLabelsGeneratorMixin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYLabelsGeneratorMixin, qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYLabelsGeneratorMixin::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27JKQTPXYLabelsGeneratorMixinE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
