/****************************************************************************
** Meta object code from reading C++ file 'jkqtplotter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtplotter.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtplotter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11JKQTPlotterE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPlotter::qt_create_metaobjectdata<qt_meta_tag_ZN11JKQTPlotterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPlotter",
        "plotMouseMove",
        "",
        "x",
        "y",
        "plotMouseClicked",
        "Qt::KeyboardModifiers",
        "modifiers",
        "Qt::MouseButton",
        "button",
        "plotMouseDoubleClicked",
        "plotMouseWheelOperated",
        "deltaAngleX",
        "deltaAngleY",
        "plotNewZoomRectangle",
        "mouseDragRectXStart",
        "mouseDragRectXEnd",
        "mouseDragRectYStart",
        "mouseDragRectYEnd",
        "plotScalingRecalculated",
        "beforePlotScalingRecalculate",
        "contextMenuOpened",
        "QMenu*",
        "contextMenu",
        "zoomChangedLocally",
        "newxmin",
        "newxmax",
        "newymin",
        "newymax",
        "JKQTPlotter*",
        "sender",
        "widgetResized",
        "new_width",
        "new_height",
        "userScribbleClick",
        "first",
        "last",
        "userRectangleFinished",
        "width",
        "height",
        "userLineFinished",
        "x1",
        "y1",
        "x2",
        "y2",
        "userCircleFinished",
        "radius",
        "userEllipseFinished",
        "radiusX",
        "radiusY",
        "tooltipDisplayed",
        "entries",
        "QList<JKQTPPlotElement*>",
        "graphs",
        "rulerDisplayed",
        "setMagnification",
        "m",
        "zoom",
        "xminn",
        "xmaxn",
        "yminn",
        "ymaxn",
        "affectsSecondaryAxes",
        "setGrid",
        "val",
        "setGridColor",
        "QColor",
        "color",
        "setMinorGridColor",
        "setGridWidth",
        "__value",
        "setMinorGridWidth",
        "setGridStyle",
        "Qt::PenStyle",
        "setMinorGridStyle",
        "setShowZeroAxes",
        "showX",
        "showY",
        "showXY",
        "saveImage",
        "filename",
        "displayPreview",
        "saveAsPixelImage",
        "outputFormat",
        "QSize",
        "outputSizeIncrease",
        "saveAsSVG",
        "saveAsPDF",
        "saveAsCSV",
        "saveAsSemicolonSV",
        "saveAsTabSV",
        "saveAsDIF",
        "saveAsSYLK",
        "saveAsMatlab",
        "saveAsGerExcelCSV",
        "saveData",
        "format",
        "print",
        "QPrinter*",
        "printer",
        "copyData",
        "copyDataMatlab",
        "zoomToFit",
        "zoomX",
        "zoomY",
        "includeX0",
        "includeY0",
        "scaleX",
        "scaleY",
        "zoomIn",
        "factor",
        "zoomOut",
        "redrawPlot",
        "setMouseActionToolbarActionsActive",
        "setToolbarEnabled",
        "setToolbarAlwaysOn",
        "setMousePositionShown",
        "setUserActionOverlayPen",
        "QPen",
        "setUserActionOverlayBrush",
        "QBrush",
        "setUserActionOpaquePen",
        "setUserActionOpaqueBrush",
        "setUserActionMarkerPen",
        "setUserActionMarkerBrush",
        "setMaxTooltipDistance",
        "v",
        "setUserActionCatchSensitivity",
        "setUserActionMarkerDiameter",
        "setUserActionMarkerType",
        "JKQTPUserActionMarkerType",
        "setContextMenuMode",
        "JKQTPContextMenuModes",
        "mode",
        "synchronizeXAxis",
        "synchronizeYAxis",
        "synchronizeXYAxis",
        "populateToolbar",
        "QToolBar*",
        "toolbar",
        "openContextMenu",
        "openStandardContextMenu",
        "openSpecialContextMenu",
        "openStandardAndSpecialContextMenu",
        "setAbsoluteX",
        "xmaxx",
        "setAbsoluteY",
        "ymaxx",
        "setAbsoluteXY",
        "setX",
        "setY",
        "setXY",
        "delayedResizeEvent",
        "masterPlotScalingRecalculated",
        "pzoomChangedLocally",
        "JKQTBasePlotter*",
        "intBeforePlotScalingRecalculate",
        "reactGraphVisible",
        "visible",
        "setMouseLeftActionAsPanView",
        "setMouseLeftActionAsZoomRect",
        "setMouseLeftActionAsRuler",
        "setMouseLeftDragActionAsToolTip",
        "resetMouseLeftAction",
        "setMouseMoveActionAsToolTip",
        "enabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'plotMouseMove'
        QtMocHelpers::SignalData<void(double, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Signal 'plotMouseClicked'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers, Qt::MouseButton)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'plotMouseDoubleClicked'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers, Qt::MouseButton)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'plotMouseWheelOperated'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers, int, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { 0x80000000 | 6, 7 }, { QMetaType::Int, 12 },
            { QMetaType::Int, 13 },
        }}),
        // Signal 'plotNewZoomRectangle'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 15 }, { QMetaType::Double, 16 }, { QMetaType::Double, 17 }, { QMetaType::Double, 18 },
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'plotScalingRecalculated'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beforePlotScalingRecalculate'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contextMenuOpened'
        QtMocHelpers::SignalData<void(double, double, QMenu *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { 0x80000000 | 22, 23 },
        }}),
        // Signal 'zoomChangedLocally'
        QtMocHelpers::SignalData<void(double, double, double, double, JKQTPlotter *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 }, { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
            { 0x80000000 | 29, 30 },
        }}),
        // Signal 'widgetResized'
        QtMocHelpers::SignalData<void(int, int, JKQTPlotter *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Int, 33 }, { 0x80000000 | 29, 30 },
        }}),
        // Signal 'userScribbleClick'
        QtMocHelpers::SignalData<void(double, double, Qt::KeyboardModifiers, bool, bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { 0x80000000 | 6, 7 }, { QMetaType::Bool, 35 },
            { QMetaType::Bool, 36 },
        }}),
        // Signal 'userRectangleFinished'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { QMetaType::Double, 38 }, { QMetaType::Double, 39 },
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'userLineFinished'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 41 }, { QMetaType::Double, 42 }, { QMetaType::Double, 43 }, { QMetaType::Double, 44 },
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'userCircleFinished'
        QtMocHelpers::SignalData<void(double, double, double, Qt::KeyboardModifiers)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { QMetaType::Double, 46 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'userEllipseFinished'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { QMetaType::Double, 48 }, { QMetaType::Double, 49 },
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'tooltipDisplayed'
        QtMocHelpers::SignalData<void(double, double, const QStringList &, const QList<JKQTPPlotElement*> &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 }, { QMetaType::QStringList, 51 }, { 0x80000000 | 52, 53 },
        }}),
        // Signal 'rulerDisplayed'
        QtMocHelpers::SignalData<void(double, double, double, double, Qt::KeyboardModifiers)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 41 }, { QMetaType::Double, 42 }, { QMetaType::Double, 43 }, { QMetaType::Double, 44 },
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setMagnification'
        QtMocHelpers::SlotData<void(double)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 56 },
        }}),
        // Slot 'zoom'
        QtMocHelpers::SlotData<void(double, double, double, double, bool)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 59 }, { QMetaType::Double, 60 }, { QMetaType::Double, 61 },
            { QMetaType::Bool, 62 },
        }}),
        // Slot 'zoom'
        QtMocHelpers::SlotData<void(double, double, double, double)>(57, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 59 }, { QMetaType::Double, 60 }, { QMetaType::Double, 61 },
        }}),
        // Slot 'setGrid'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 64 },
        }}),
        // Slot 'setGridColor'
        QtMocHelpers::SlotData<void(QColor)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 66, 67 },
        }}),
        // Slot 'setMinorGridColor'
        QtMocHelpers::SlotData<void(QColor)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 66, 67 },
        }}),
        // Slot 'setGridWidth'
        QtMocHelpers::SlotData<void(double)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 70 },
        }}),
        // Slot 'setMinorGridWidth'
        QtMocHelpers::SlotData<void(double)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 70 },
        }}),
        // Slot 'setGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 73, 70 },
        }}),
        // Slot 'setMinorGridStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 73, 70 },
        }}),
        // Slot 'setShowZeroAxes'
        QtMocHelpers::SlotData<void(bool, bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 76 }, { QMetaType::Bool, 77 },
        }}),
        // Slot 'setShowZeroAxes'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 78 },
        }}),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(79, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 },
        }}),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool(const QString &)>(79, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveImage'
        QtMocHelpers::SlotData<bool()>(79, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool, const QByteArray &, const QSize &)>(82, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 }, { QMetaType::QByteArray, 83 }, { 0x80000000 | 84, 85 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool, const QByteArray &)>(82, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 }, { QMetaType::QByteArray, 83 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(82, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool(const QString &)>(82, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsPixelImage'
        QtMocHelpers::SlotData<bool()>(82, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(86, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 },
        }}),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool(const QString &)>(86, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsSVG'
        QtMocHelpers::SlotData<bool()>(86, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool(const QString &, bool)>(87, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 80 }, { QMetaType::Bool, 81 },
        }}),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool(const QString &)>(87, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsPDF'
        QtMocHelpers::SlotData<bool()>(87, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'saveAsCSV'
        QtMocHelpers::SlotData<void(const QString &)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsCSV'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsSemicolonSV'
        QtMocHelpers::SlotData<void(const QString &)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsSemicolonSV'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsTabSV'
        QtMocHelpers::SlotData<void(const QString &)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsTabSV'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsDIF'
        QtMocHelpers::SlotData<void(const QString &)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsDIF'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsSYLK'
        QtMocHelpers::SlotData<void(const QString &)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsSYLK'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsMatlab'
        QtMocHelpers::SlotData<void(const QString &)>(93, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsMatlab'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveAsGerExcelCSV'
        QtMocHelpers::SlotData<void(const QString &)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveAsGerExcelCSV'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveData'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 80 }, { QMetaType::QString, 96 },
        }}),
        // Slot 'saveData'
        QtMocHelpers::SlotData<void(const QString &)>(95, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 80 },
        }}),
        // Slot 'saveData'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void(QPrinter *)>(97, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 98, 99 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copyData'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyDataMatlab'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool, double, double)>(102, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 103 }, { QMetaType::Bool, 104 }, { QMetaType::Bool, 105 }, { QMetaType::Bool, 106 },
            { QMetaType::Double, 107 }, { QMetaType::Double, 108 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool, double)>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 103 }, { QMetaType::Bool, 104 }, { QMetaType::Bool, 105 }, { QMetaType::Bool, 106 },
            { QMetaType::Double, 107 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool, bool)>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 103 }, { QMetaType::Bool, 104 }, { QMetaType::Bool, 105 }, { QMetaType::Bool, 106 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool, bool)>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 103 }, { QMetaType::Bool, 104 }, { QMetaType::Bool, 105 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool, bool)>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 103 }, { QMetaType::Bool, 104 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void(bool)>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 103 },
        }}),
        // Slot 'zoomToFit'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void(double)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 110 },
        }}),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void(double)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 110 },
        }}),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'redrawPlot'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMouseActionToolbarActionsActive'
        QtMocHelpers::SlotData<void(bool)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 70 },
        }}),
        // Slot 'setToolbarEnabled'
        QtMocHelpers::SlotData<void(bool)>(114, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 70 },
        }}),
        // Slot 'setToolbarAlwaysOn'
        QtMocHelpers::SlotData<void(bool)>(115, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 70 },
        }}),
        // Slot 'setMousePositionShown'
        QtMocHelpers::SlotData<void(bool)>(116, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 70 },
        }}),
        // Slot 'setUserActionOverlayPen'
        QtMocHelpers::SlotData<void(const QPen &)>(117, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 118, 70 },
        }}),
        // Slot 'setUserActionOverlayBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(119, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 120, 70 },
        }}),
        // Slot 'setUserActionOpaquePen'
        QtMocHelpers::SlotData<void(const QPen &)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 118, 70 },
        }}),
        // Slot 'setUserActionOpaqueBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(122, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 120, 70 },
        }}),
        // Slot 'setUserActionMarkerPen'
        QtMocHelpers::SlotData<void(const QPen &)>(123, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 118, 70 },
        }}),
        // Slot 'setUserActionMarkerBrush'
        QtMocHelpers::SlotData<void(const QBrush &)>(124, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 120, 70 },
        }}),
        // Slot 'setMaxTooltipDistance'
        QtMocHelpers::SlotData<void(int)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 126 },
        }}),
        // Slot 'setUserActionCatchSensitivity'
        QtMocHelpers::SlotData<void(int)>(127, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 126 },
        }}),
        // Slot 'setUserActionMarkerDiameter'
        QtMocHelpers::SlotData<void(int)>(128, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 126 },
        }}),
        // Slot 'setUserActionMarkerType'
        QtMocHelpers::SlotData<void(JKQTPUserActionMarkerType)>(129, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 130, 126 },
        }}),
        // Slot 'setContextMenuMode'
        QtMocHelpers::SlotData<void(JKQTPContextMenuModes)>(131, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 132, 133 },
        }}),
        // Slot 'synchronizeXAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTPlotter *)>(134, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 }, { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'synchronizeYAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTPlotter *)>(135, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 }, { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'synchronizeXYAxis'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTPlotter *)>(136, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 }, { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'populateToolbar'
        QtMocHelpers::SlotData<void(QToolBar *) const>(137, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 138, 139 },
        }}),
        // Slot 'openContextMenu'
        QtMocHelpers::SlotData<void()>(140, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openContextMenu'
        QtMocHelpers::SlotData<void(int, int)>(140, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'openStandardContextMenu'
        QtMocHelpers::SlotData<void()>(141, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openStandardContextMenu'
        QtMocHelpers::SlotData<void(int, int)>(141, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'openSpecialContextMenu'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openSpecialContextMenu'
        QtMocHelpers::SlotData<void(int, int)>(142, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'openStandardAndSpecialContextMenu'
        QtMocHelpers::SlotData<void()>(143, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openStandardAndSpecialContextMenu'
        QtMocHelpers::SlotData<void(int, int)>(143, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'setAbsoluteX'
        QtMocHelpers::SlotData<void(double, double)>(144, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 145 },
        }}),
        // Slot 'setAbsoluteY'
        QtMocHelpers::SlotData<void(double, double)>(146, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 60 }, { QMetaType::Double, 147 },
        }}),
        // Slot 'setAbsoluteXY'
        QtMocHelpers::SlotData<void(double, double, double, double)>(148, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 145 }, { QMetaType::Double, 60 }, { QMetaType::Double, 147 },
        }}),
        // Slot 'setX'
        QtMocHelpers::SlotData<void(double, double)>(149, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 145 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(double, double)>(150, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 60 }, { QMetaType::Double, 147 },
        }}),
        // Slot 'setXY'
        QtMocHelpers::SlotData<void(double, double, double, double, bool)>(151, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 145 }, { QMetaType::Double, 60 }, { QMetaType::Double, 147 },
            { QMetaType::Bool, 62 },
        }}),
        // Slot 'setXY'
        QtMocHelpers::SlotData<void(double, double, double, double)>(151, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 58 }, { QMetaType::Double, 145 }, { QMetaType::Double, 60 }, { QMetaType::Double, 147 },
        }}),
        // Slot 'delayedResizeEvent'
        QtMocHelpers::SlotData<void()>(152, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'masterPlotScalingRecalculated'
        QtMocHelpers::SlotData<void()>(153, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'pzoomChangedLocally'
        QtMocHelpers::SlotData<void(double, double, double, double, JKQTBasePlotter *)>(154, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 25 }, { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 28 },
            { 0x80000000 | 155, 30 },
        }}),
        // Slot 'intBeforePlotScalingRecalculate'
        QtMocHelpers::SlotData<void()>(156, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reactGraphVisible'
        QtMocHelpers::SlotData<void(bool)>(157, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 158 },
        }}),
        // Slot 'setMouseLeftActionAsPanView'
        QtMocHelpers::SlotData<void()>(159, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setMouseLeftActionAsZoomRect'
        QtMocHelpers::SlotData<void()>(160, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setMouseLeftActionAsRuler'
        QtMocHelpers::SlotData<void()>(161, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setMouseLeftDragActionAsToolTip'
        QtMocHelpers::SlotData<void()>(162, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'resetMouseLeftAction'
        QtMocHelpers::SlotData<void()>(163, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setMouseMoveActionAsToolTip'
        QtMocHelpers::SlotData<void(bool)>(164, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 165 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPlotter, qt_meta_tag_ZN11JKQTPlotterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPlotter::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPlotterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPlotterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11JKQTPlotterE_t>.metaTypes,
    nullptr
} };

void JKQTPlotter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPlotter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->plotMouseMove((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: _t->plotMouseClicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[4]))); break;
        case 2: _t->plotMouseDoubleClicked((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Qt::MouseButton>>(_a[4]))); break;
        case 3: _t->plotMouseWheelOperated((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 4: _t->plotNewZoomRectangle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 5: _t->plotScalingRecalculated(); break;
        case 6: _t->beforePlotScalingRecalculate(); break;
        case 7: _t->contextMenuOpened((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[3]))); break;
        case 8: _t->zoomChangedLocally((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTPlotter*>>(_a[5]))); break;
        case 9: _t->widgetResized((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<JKQTPlotter*>>(_a[3]))); break;
        case 10: _t->userScribbleClick((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 11: _t->userRectangleFinished((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 12: _t->userLineFinished((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 13: _t->userCircleFinished((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[4]))); break;
        case 14: _t->userEllipseFinished((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 15: _t->tooltipDisplayed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QList<JKQTPPlotElement*>>>(_a[4]))); break;
        case 16: _t->rulerDisplayed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[5]))); break;
        case 17: _t->setMagnification((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->zoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 19: _t->zoom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 20: _t->setGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 22: _t->setMinorGridColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 23: _t->setGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->setMinorGridWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->setGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 26: _t->setMinorGridStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 27: _t->setShowZeroAxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 28: _t->setShowZeroAxes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->saveImage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->saveAsPixelImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->saveAsPixelImage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: { bool _r = _t->saveAsSVG((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->saveAsSVG((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->saveAsSVG();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 40: { bool _r = _t->saveAsPDF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: { bool _r = _t->saveAsPDF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 42: { bool _r = _t->saveAsPDF();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->saveAsCSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 44: _t->saveAsCSV(); break;
        case 45: _t->saveAsSemicolonSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: _t->saveAsSemicolonSV(); break;
        case 47: _t->saveAsTabSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 48: _t->saveAsTabSV(); break;
        case 49: _t->saveAsDIF((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 50: _t->saveAsDIF(); break;
        case 51: _t->saveAsSYLK((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 52: _t->saveAsSYLK(); break;
        case 53: _t->saveAsMatlab((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 54: _t->saveAsMatlab(); break;
        case 55: _t->saveAsGerExcelCSV((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 56: _t->saveAsGerExcelCSV(); break;
        case 57: _t->saveData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 58: _t->saveData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 59: _t->saveData(); break;
        case 60: _t->print((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 61: _t->print(); break;
        case 62: _t->copyData(); break;
        case 63: _t->copyDataMatlab(); break;
        case 64: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6]))); break;
        case 65: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5]))); break;
        case 66: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 67: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 68: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 69: _t->zoomToFit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->zoomToFit(); break;
        case 71: _t->zoomIn((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 72: _t->zoomIn(); break;
        case 73: _t->zoomOut((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 74: _t->zoomOut(); break;
        case 75: _t->redrawPlot(); break;
        case 76: _t->setMouseActionToolbarActionsActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: _t->setToolbarEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->setToolbarAlwaysOn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 79: _t->setMousePositionShown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 80: _t->setUserActionOverlayPen((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 81: _t->setUserActionOverlayBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 82: _t->setUserActionOpaquePen((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 83: _t->setUserActionOpaqueBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 84: _t->setUserActionMarkerPen((*reinterpret_cast<std::add_pointer_t<QPen>>(_a[1]))); break;
        case 85: _t->setUserActionMarkerBrush((*reinterpret_cast<std::add_pointer_t<QBrush>>(_a[1]))); break;
        case 86: _t->setMaxTooltipDistance((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 87: _t->setUserActionCatchSensitivity((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 88: _t->setUserActionMarkerDiameter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 89: _t->setUserActionMarkerType((*reinterpret_cast<std::add_pointer_t<JKQTPUserActionMarkerType>>(_a[1]))); break;
        case 90: _t->setContextMenuMode((*reinterpret_cast<std::add_pointer_t<JKQTPContextMenuModes>>(_a[1]))); break;
        case 91: _t->synchronizeXAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTPlotter*>>(_a[5]))); break;
        case 92: _t->synchronizeYAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTPlotter*>>(_a[5]))); break;
        case 93: _t->synchronizeXYAxis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTPlotter*>>(_a[5]))); break;
        case 94: _t->populateToolbar((*reinterpret_cast<std::add_pointer_t<QToolBar*>>(_a[1]))); break;
        case 95: _t->openContextMenu(); break;
        case 96: _t->openContextMenu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 97: _t->openStandardContextMenu(); break;
        case 98: _t->openStandardContextMenu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 99: _t->openSpecialContextMenu(); break;
        case 100: _t->openSpecialContextMenu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 101: _t->openStandardAndSpecialContextMenu(); break;
        case 102: _t->openStandardAndSpecialContextMenu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 103: _t->setAbsoluteX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 104: _t->setAbsoluteY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 105: _t->setAbsoluteXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 106: _t->setX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 107: _t->setY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 108: _t->setXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 109: _t->setXY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 110: _t->delayedResizeEvent(); break;
        case 111: _t->masterPlotScalingRecalculated(); break;
        case 112: _t->pzoomChangedLocally((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<JKQTBasePlotter*>>(_a[5]))); break;
        case 113: _t->intBeforePlotScalingRecalculate(); break;
        case 114: _t->reactGraphVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 115: _t->setMouseLeftActionAsPanView(); break;
        case 116: _t->setMouseLeftActionAsZoomRect(); break;
        case 117: _t->setMouseLeftActionAsRuler(); break;
        case 118: _t->setMouseLeftDragActionAsToolTip(); break;
        case 119: _t->resetMouseLeftAction(); break;
        case 120: _t->setMouseMoveActionAsToolTip((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTPlotter* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTPlotter* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<JKQTPPlotElement*> >(); break;
            }
            break;
        case 91:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTPlotter* >(); break;
            }
            break;
        case 92:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTPlotter* >(); break;
            }
            break;
        case 93:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTPlotter* >(); break;
            }
            break;
        case 94:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QToolBar* >(); break;
            }
            break;
        case 112:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< JKQTBasePlotter* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double )>(_a, &JKQTPlotter::plotMouseMove, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , Qt::KeyboardModifiers , Qt::MouseButton )>(_a, &JKQTPlotter::plotMouseClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , Qt::KeyboardModifiers , Qt::MouseButton )>(_a, &JKQTPlotter::plotMouseDoubleClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , Qt::KeyboardModifiers , int , int )>(_a, &JKQTPlotter::plotMouseWheelOperated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::plotNewZoomRectangle, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)()>(_a, &JKQTPlotter::plotScalingRecalculated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)()>(_a, &JKQTPlotter::beforePlotScalingRecalculate, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , QMenu * )>(_a, &JKQTPlotter::contextMenuOpened, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , JKQTPlotter * )>(_a, &JKQTPlotter::zoomChangedLocally, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(int , int , JKQTPlotter * )>(_a, &JKQTPlotter::widgetResized, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , Qt::KeyboardModifiers , bool , bool )>(_a, &JKQTPlotter::userScribbleClick, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::userRectangleFinished, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::userLineFinished, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::userCircleFinished, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::userEllipseFinished, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , const QStringList & , const QList<JKQTPPlotElement*> & )>(_a, &JKQTPlotter::tooltipDisplayed, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (JKQTPlotter::*)(double , double , double , double , Qt::KeyboardModifiers )>(_a, &JKQTPlotter::rulerDisplayed, 16))
            return;
    }
}

const QMetaObject *JKQTPlotter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPlotter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPlotterE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int JKQTPlotter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 121)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 121;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 121)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 121;
    }
    return _id;
}

// SIGNAL 0
void JKQTPlotter::plotMouseMove(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void JKQTPlotter::plotMouseClicked(double _t1, double _t2, Qt::KeyboardModifiers _t3, Qt::MouseButton _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void JKQTPlotter::plotMouseDoubleClicked(double _t1, double _t2, Qt::KeyboardModifiers _t3, Qt::MouseButton _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void JKQTPlotter::plotMouseWheelOperated(double _t1, double _t2, Qt::KeyboardModifiers _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 4
void JKQTPlotter::plotNewZoomRectangle(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 5
void JKQTPlotter::plotScalingRecalculated()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void JKQTPlotter::beforePlotScalingRecalculate()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void JKQTPlotter::contextMenuOpened(double _t1, double _t2, QMenu * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void JKQTPlotter::zoomChangedLocally(double _t1, double _t2, double _t3, double _t4, JKQTPlotter * _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 9
void JKQTPlotter::widgetResized(int _t1, int _t2, JKQTPlotter * _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3);
}

// SIGNAL 10
void JKQTPlotter::userScribbleClick(double _t1, double _t2, Qt::KeyboardModifiers _t3, bool _t4, bool _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 11
void JKQTPlotter::userRectangleFinished(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 12
void JKQTPlotter::userLineFinished(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 13
void JKQTPlotter::userCircleFinished(double _t1, double _t2, double _t3, Qt::KeyboardModifiers _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 14
void JKQTPlotter::userEllipseFinished(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 15
void JKQTPlotter::tooltipDisplayed(double _t1, double _t2, const QStringList & _t3, const QList<JKQTPPlotElement*> & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 16
void JKQTPlotter::rulerDisplayed(double _t1, double _t2, double _t3, double _t4, Qt::KeyboardModifiers _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2, _t3, _t4, _t5);
}
QT_WARNING_POP
