/****************************************************************************
** Meta object code from reading C++ file 'jkqtpfinancial.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpfinancial.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpfinancial.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19JKQTPFinancialGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFinancialGraph::qt_create_metaobjectdata<qt_meta_tag_ZN19JKQTPFinancialGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFinancialGraph",
        "autoscaleBoxWidthAndShift",
        "",
        "maxWidth",
        "shrinkFactor",
        "autoscaleBoxWidthAndShiftSeparatedGroups",
        "groupWidth",
        "setShift",
        "__value",
        "setWidth",
        "setGraphType",
        "FinancialGraphType",
        "setColor",
        "QColor",
        "cPositive",
        "cNegative",
        "linePositive",
        "fillPositive",
        "lineNegative",
        "fillNegative",
        "setCandlestickTwoColor",
        "lineCOlor",
        "setCandlestickOneColor",
        "cLine",
        "setOHLCTwoColor",
        "setOpenColumn",
        "size_t",
        "setCloseColumn",
        "setHighColumn",
        "setLowColumn",
        "openColumn",
        "closeColumn",
        "highColumn",
        "lowColumn",
        "graphType",
        "CandleStick",
        "OHLC"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'autoscaleBoxWidthAndShift'
        QtMocHelpers::SlotData<void(double, double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::Double, 4 },
        }}),
        // Slot 'autoscaleBoxWidthAndShift'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'autoscaleBoxWidthAndShift'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'autoscaleBoxWidthAndShiftSeparatedGroups'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'autoscaleBoxWidthAndShiftSeparatedGroups'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setShift'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setWidth'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setGraphType'
        QtMocHelpers::SlotData<void(enum FinancialGraphType)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 8 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor, QColor)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor, QColor, QColor, QColor)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 16 }, { 0x80000000 | 13, 17 }, { 0x80000000 | 13, 18 }, { 0x80000000 | 13, 19 },
        }}),
        // Slot 'setCandlestickTwoColor'
        QtMocHelpers::SlotData<void(QColor, QColor)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'setCandlestickTwoColor'
        QtMocHelpers::SlotData<void(QColor, QColor, QColor)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 }, { 0x80000000 | 13, 21 },
        }}),
        // Slot 'setCandlestickOneColor'
        QtMocHelpers::SlotData<void(QColor)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 23 },
        }}),
        // Slot 'setOHLCTwoColor'
        QtMocHelpers::SlotData<void(QColor, QColor)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'setOpenColumn'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setOpenColumn'
        QtMocHelpers::SlotData<void(size_t)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 8 },
        }}),
        // Slot 'setCloseColumn'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setCloseColumn'
        QtMocHelpers::SlotData<void(size_t)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 8 },
        }}),
        // Slot 'setHighColumn'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setHighColumn'
        QtMocHelpers::SlotData<void(size_t)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 8 },
        }}),
        // Slot 'setLowColumn'
        QtMocHelpers::SlotData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setLowColumn'
        QtMocHelpers::SlotData<void(size_t)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'openColumn'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'closeColumn'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'highColumn'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'lowColumn'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'graphType'
        QtMocHelpers::PropertyData<enum FinancialGraphType>(34, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FinancialGraphType'
        QtMocHelpers::EnumData<enum FinancialGraphType>(11, 11, QMC::EnumFlags{}).add({
            {   35, FinancialGraphType::CandleStick },
            {   36, FinancialGraphType::OHLC },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPFinancialGraph, qt_meta_tag_ZN19JKQTPFinancialGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFinancialGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPFinancialGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPFinancialGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19JKQTPFinancialGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPFinancialGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFinancialGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->autoscaleBoxWidthAndShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 1: _t->autoscaleBoxWidthAndShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->autoscaleBoxWidthAndShift(); break;
        case 3: _t->autoscaleBoxWidthAndShiftSeparatedGroups((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->autoscaleBoxWidthAndShiftSeparatedGroups(); break;
        case 5: _t->setShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->setWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setGraphType((*reinterpret_cast<std::add_pointer_t<enum FinancialGraphType>>(_a[1]))); break;
        case 8: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 9: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[4]))); break;
        case 10: _t->setCandlestickTwoColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 11: _t->setCandlestickTwoColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[3]))); break;
        case 12: _t->setCandlestickOneColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->setOHLCTwoColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[2]))); break;
        case 14: _t->setOpenColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->setOpenColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 16: _t->setCloseColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->setCloseColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 18: _t->setHighColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->setHighColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 20: _t->setLowColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->setLowColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getOpenColumn(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getCloseColumn(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getHighColumn(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->getLowColumn(); break;
        case 4: *reinterpret_cast<enum FinancialGraphType*>(_v) = _t->getGraphType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOpenColumn(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setCloseColumn(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setHighColumn(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setLowColumn(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setGraphType(*reinterpret_cast<enum FinancialGraphType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPFinancialGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFinancialGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPFinancialGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXGraph::qt_metacast(_clname);
}

int JKQTPFinancialGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
