/****************************************************************************
** Meta object code from reading C++ file 'jkqtprange.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtprange.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtprange.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14JKQTPRangeBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPRangeBase::qt_create_metaobjectdata<qt_meta_tag_ZN14JKQTPRangeBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPRangeBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setDrawCenterLineOnly",
        "setCenterColor",
        "__value",
        "setCenterStyle",
        "Qt::PenStyle",
        "setPlotRangeLines",
        "setFillRange",
        "setPlotRange",
        "setInvertedRange",
        "setRangeCenter",
        "setPlotCenterLine",
        "setUnlimitedSizeMax",
        "setUnlimitedSizeMin",
        "setSizeMax",
        "setSizeMin",
        "setRangeMax",
        "setRangeMin"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDrawCenterLineOnly'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCenterColor'
        QtMocHelpers::SlotData<void(const QColor &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 },
        }}),
        // Slot 'setCenterStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setPlotRangeLines'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setFillRange'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setPlotRange'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setInvertedRange'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setRangeCenter'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setPlotCenterLine'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setUnlimitedSizeMax'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setUnlimitedSizeMin'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'setSizeMax'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setSizeMin'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setRangeMax'
        QtMocHelpers::SlotData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setRangeMin'
        QtMocHelpers::SlotData<void(double)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPRangeBase, qt_meta_tag_ZN14JKQTPRangeBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPRangeBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPRangeBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPRangeBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14JKQTPRangeBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPRangeBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPRangeBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setDrawCenterLineOnly(); break;
        case 2: _t->setCenterColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->setCenterStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 4: _t->setPlotRangeLines((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setFillRange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setPlotRange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setInvertedRange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setRangeCenter((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->setPlotCenterLine((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setUnlimitedSizeMax((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setUnlimitedSizeMin((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setSizeMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->setSizeMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->setRangeMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->setRangeMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPRangeBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPRangeBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14JKQTPRangeBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineStyleMixin"))
        return static_cast< JKQTPGraphLineStyleMixin*>(this);
    if (!strcmp(_clname, "JKQTPGraphFillStyleMixin"))
        return static_cast< JKQTPGraphFillStyleMixin*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPRangeBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN20JKQTPHorizontalRangeE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPHorizontalRange::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPHorizontalRange"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPHorizontalRange, qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPHorizontalRange::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPRangeBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>.metaTypes,
    nullptr
} };

void JKQTPHorizontalRange::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPHorizontalRange *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPHorizontalRange::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPHorizontalRange::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPHorizontalRangeE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPRangeBase::qt_metacast(_clname);
}

int JKQTPHorizontalRange::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPRangeBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18JKQTPVerticalRangeE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPVerticalRange::qt_create_metaobjectdata<qt_meta_tag_ZN18JKQTPVerticalRangeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPVerticalRange"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPVerticalRange, qt_meta_tag_ZN18JKQTPVerticalRangeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPVerticalRange::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPRangeBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPVerticalRangeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPVerticalRangeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18JKQTPVerticalRangeE_t>.metaTypes,
    nullptr
} };

void JKQTPVerticalRange::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPVerticalRange *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPVerticalRange::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPVerticalRange::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JKQTPVerticalRangeE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPRangeBase::qt_metacast(_clname);
}

int JKQTPVerticalRange::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPRangeBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
