/****************************************************************************
** Meta object code from reading C++ file 'jkqtpviolinplot.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpviolinplot.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpviolinplot.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPViolinplotElementBase::qt_create_metaobjectdata<qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPViolinplotElementBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setViolinFrequencyColumn",
        "__value",
        "size_t",
        "setViolinPositionColumn",
        "setDrawMinMax",
        "setDrawMedian",
        "setDrawMean",
        "setMax",
        "setMin",
        "setMean",
        "setMedian"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setViolinFrequencyColumn'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setViolinFrequencyColumn'
        QtMocHelpers::SlotData<void(size_t)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 6 },
        }}),
        // Slot 'setViolinPositionColumn'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setViolinPositionColumn'
        QtMocHelpers::SlotData<void(size_t)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 6 },
        }}),
        // Slot 'setDrawMinMax'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawMedian'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawMean'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setMax'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMin'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMean'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setMedian'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPViolinplotElementBase, qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPViolinplotElementBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPViolinplotElementBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPViolinplotElementBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setViolinFrequencyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setViolinFrequencyColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 3: _t->setViolinPositionColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setViolinPositionColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 5: _t->setDrawMinMax((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setDrawMedian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setDrawMean((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->setMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->setMean((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->setMedian((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPViolinplotElementBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPViolinplotElementBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPViolinplotElementBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphViolinplotStyleMixin"))
        return static_cast< JKQTPGraphViolinplotStyleMixin*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPViolinplotElementBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPViolinplotVerticalElement::qt_create_metaobjectdata<qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPViolinplotVerticalElement"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPViolinplotVerticalElement, qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPViolinplotVerticalElement::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPViolinplotElementBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>.metaTypes,
    nullptr
} };

void JKQTPViolinplotVerticalElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPViolinplotVerticalElement *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPViolinplotVerticalElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPViolinplotVerticalElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30JKQTPViolinplotVerticalElementE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPViolinplotElementBase::qt_metacast(_clname);
}

int JKQTPViolinplotVerticalElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPViolinplotElementBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPViolinplotHorizontalElement::qt_create_metaobjectdata<qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPViolinplotHorizontalElement"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPViolinplotHorizontalElement, qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPViolinplotHorizontalElement::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPViolinplotElementBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>.metaTypes,
    nullptr
} };

void JKQTPViolinplotHorizontalElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPViolinplotHorizontalElement *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPViolinplotHorizontalElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPViolinplotHorizontalElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32JKQTPViolinplotHorizontalElementE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPViolinplotElementBase::qt_metacast(_clname);
}

int JKQTPViolinplotHorizontalElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPViolinplotElementBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
