/****************************************************************************
** Meta object code from reading C++ file 'jkqtpspecialline.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpspecialline.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpspecialline.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPSpecialLineGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPSpecialLineGraphBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setDrawSymbols",
        "__value",
        "setSpecialLineType",
        "JKQTPSpecialLineType",
        "drawSymbols",
        "specialLineType"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDrawSymbols'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setSpecialLineType'
        QtMocHelpers::SlotData<void(const JKQTPSpecialLineType &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'drawSymbols'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'specialLineType'
        QtMocHelpers::PropertyData<JKQTPSpecialLineType>(10, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPSpecialLineGraphBase, qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPSpecialLineGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYBaselineGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPSpecialLineGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPSpecialLineGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setDrawSymbols((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setSpecialLineType((*reinterpret_cast<std::add_pointer_t<JKQTPSpecialLineType>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getDrawSymbols(); break;
        case 1: *reinterpret_cast<JKQTPSpecialLineType*>(_v) = _t->getSpecialLineType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDrawSymbols(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setSpecialLineType(*reinterpret_cast<JKQTPSpecialLineType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPSpecialLineGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPSpecialLineGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPSpecialLineGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineAndFillStyleMixin"))
        return static_cast< JKQTPGraphLineAndFillStyleMixin*>(this);
    if (!strcmp(_clname, "JKQTPGraphSymbolStyleMixin"))
        return static_cast< JKQTPGraphSymbolStyleMixin*>(this);
    if (!strcmp(_clname, "JKQTPGraphLinesCompressionMixin"))
        return static_cast< JKQTPGraphLinesCompressionMixin*>(this);
    return JKQTPXYBaselineGraph::qt_metacast(_clname);
}

int JKQTPSpecialLineGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYBaselineGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPSpecialLineHorizontalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPSpecialLineHorizontalGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPSpecialLineHorizontalGraph, qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPSpecialLineHorizontalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPSpecialLineGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPSpecialLineHorizontalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPSpecialLineHorizontalGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPSpecialLineHorizontalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPSpecialLineHorizontalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPSpecialLineHorizontalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPSpecialLineGraphBase::qt_metacast(_clname);
}

int JKQTPSpecialLineHorizontalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPSpecialLineGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPSpecialLineVerticalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPSpecialLineVerticalGraph",
        "setKeyColumn",
        "",
        "__value",
        "setValueColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setValueColumn'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPSpecialLineVerticalGraph, qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPSpecialLineVerticalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPSpecialLineGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPSpecialLineVerticalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPSpecialLineVerticalGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setValueColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPSpecialLineVerticalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPSpecialLineVerticalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29JKQTPSpecialLineVerticalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPSpecialLineGraphBase::qt_metacast(_clname);
}

int JKQTPSpecialLineVerticalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPSpecialLineGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
