/****************************************************************************
** Meta object code from reading C++ file 'jkqtpgraphsbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpgraphsbase.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpgraphsbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16JKQTPPlotElementE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPPlotElement::qt_create_metaobjectdata<qt_meta_tag_ZN16JKQTPPlotElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPPlotElement",
        "setVisible",
        "",
        "__value",
        "setHighlighted",
        "setTitle",
        "visible",
        "title",
        "highlighted",
        "xAxisRef",
        "JKQTPCoordinateAxisRef",
        "yAxisRef",
        "HitTestMode",
        "HitTestXY",
        "HitTestXOnly",
        "HitTestYOnly"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setVisible'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setHighlighted'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(6, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'highlighted'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'xAxisRef'
        QtMocHelpers::PropertyData<JKQTPCoordinateAxisRef>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'yAxisRef'
        QtMocHelpers::PropertyData<JKQTPCoordinateAxisRef>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'HitTestMode'
        QtMocHelpers::EnumData<enum HitTestMode>(12, 12, QMC::EnumFlags{}).add({
            {   13, HitTestMode::HitTestXY },
            {   14, HitTestMode::HitTestXOnly },
            {   15, HitTestMode::HitTestYOnly },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPPlotElement, qt_meta_tag_ZN16JKQTPPlotElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPPlotElement::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTPPlotElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTPPlotElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JKQTPPlotElementE_t>.metaTypes,
    nullptr
} };

void JKQTPPlotElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPPlotElement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setHighlighted((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isVisible(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getTitle(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isHighlighted(); break;
        case 3: *reinterpret_cast<JKQTPCoordinateAxisRef*>(_v) = _t->getXAxisRef(); break;
        case 4: *reinterpret_cast<JKQTPCoordinateAxisRef*>(_v) = _t->getYAxisRef(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setHighlighted(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setXAxis(*reinterpret_cast<JKQTPCoordinateAxisRef*>(_v)); break;
        case 4: _t->setYAxis(*reinterpret_cast<JKQTPCoordinateAxisRef*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPPlotElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPPlotElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JKQTPPlotElementE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int JKQTPPlotElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10JKQTPGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGraph::qt_create_metaobjectdata<qt_meta_tag_ZN10JKQTPGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGraph, qt_meta_tag_ZN10JKQTPGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10JKQTPGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10JKQTPGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeometricPlotElement::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeometricPlotElement",
        "setDrawMode",
        "",
        "DrawMode",
        "mode",
        "drawMode",
        "DrawAsGraphicElement",
        "DrawAsMathematicalCurve"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setDrawMode'
        QtMocHelpers::SlotData<void(enum DrawMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'drawMode'
        QtMocHelpers::PropertyData<enum DrawMode>(5, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DrawMode'
        QtMocHelpers::EnumData<enum DrawMode>(3, 3, QMC::EnumFlags{}).add({
            {    6, DrawMode::DrawAsGraphicElement },
            {    7, DrawMode::DrawAsMathematicalCurve },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPGeometricPlotElement, qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeometricPlotElement::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>.metaTypes,
    nullptr
} };

void JKQTPGeometricPlotElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeometricPlotElement *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setDrawMode((*reinterpret_cast<std::add_pointer_t<enum DrawMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum DrawMode*>(_v) = _t->getDrawMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDrawMode(*reinterpret_cast<enum DrawMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPGeometricPlotElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeometricPlotElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPGeometricPlotElementE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPGeometricPlotElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPPlotAnnotationElement::qt_create_metaobjectdata<qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPPlotAnnotationElement"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPPlotAnnotationElement, qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPPlotAnnotationElement::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPPlotElement::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>.metaTypes,
    nullptr
} };

void JKQTPPlotAnnotationElement::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPPlotAnnotationElement *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPPlotAnnotationElement::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPPlotAnnotationElement::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPPlotAnnotationElementE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPPlotElement::qt_metacast(_clname);
}

int JKQTPPlotAnnotationElement::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPPlotElement::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12JKQTPXYGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYGraph::qt_create_metaobjectdata<qt_meta_tag_ZN12JKQTPXYGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYGraph",
        "setXYColumns",
        "",
        "size_t",
        "xCol",
        "yCol",
        "std::pair<int,int>",
        "xyColPair",
        "std::pair<size_t,size_t>",
        "setDataSortOrder",
        "__value",
        "DataSortOrder",
        "setXColumn",
        "setYColumn",
        "setKeyColumn",
        "setValueColumn",
        "sortData",
        "xColumn",
        "yColumn",
        "Unsorted",
        "SortedX",
        "SortedY"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setXYColumns'
        QtMocHelpers::SlotData<void(size_t, size_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Slot 'setXYColumns'
        QtMocHelpers::SlotData<void(int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'setXYColumns'
        QtMocHelpers::SlotData<void(QPair<int,int>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setXYColumns'
        QtMocHelpers::SlotData<void(QPair<size_t,size_t>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(enum DataSortOrder)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 10 },
        }}),
        // Slot 'setXColumn'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setXColumn'
        QtMocHelpers::SlotData<void(size_t)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 },
        }}),
        // Slot 'setYColumn'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setYColumn'
        QtMocHelpers::SlotData<void(size_t)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 10 },
        }}),
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'setValueColumn'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sortData'
        QtMocHelpers::PropertyData<enum DataSortOrder>(16, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'xColumn'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'yColumn'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DataSortOrder'
        QtMocHelpers::EnumData<enum DataSortOrder>(11, 11, QMC::EnumFlags{}).add({
            {   19, DataSortOrder::Unsorted },
            {   20, DataSortOrder::SortedX },
            {   21, DataSortOrder::SortedY },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPXYGraph, qt_meta_tag_ZN12JKQTPXYGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPXYGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPXYGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12JKQTPXYGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXYGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setXYColumns((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<size_t>>(_a[2]))); break;
        case 1: _t->setXYColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->setXYColumns((*reinterpret_cast<std::add_pointer_t<std::pair<int,int>>>(_a[1]))); break;
        case 3: _t->setXYColumns((*reinterpret_cast<std::add_pointer_t<std::pair<size_t,size_t>>>(_a[1]))); break;
        case 4: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<enum DataSortOrder>>(_a[1]))); break;
        case 6: _t->setXColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setXColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 8: _t->setYColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->setYColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 10: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setValueColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum DataSortOrder*>(_v) = _t->getDataSortOrder(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getXColumn(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getYColumn(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataSortOrder(*reinterpret_cast<enum DataSortOrder*>(_v)); break;
        case 1: _t->setXColumn(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setYColumn(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXYGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPXYGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPXYGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYBaselineGraph::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYBaselineGraph",
        "setBaseline",
        "",
        "__value",
        "baseline"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setBaseline'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'baseline'
        QtMocHelpers::PropertyData<double>(4, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYBaselineGraph, qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYBaselineGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXYBaselineGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYBaselineGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setBaseline((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getBaseline(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBaseline(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXYBaselineGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYBaselineGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPXYBaselineGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXYGraph::qt_metacast(_clname);
}

int JKQTPXYBaselineGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN13JKQTPXYYGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYYGraph::qt_create_metaobjectdata<qt_meta_tag_ZN13JKQTPXYYGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYYGraph",
        "setXYYColumns",
        "",
        "size_t",
        "xCol",
        "yCol",
        "y2Col",
        "setYColumn2",
        "__value",
        "setValue2Column",
        "yColumn2"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setXYYColumns'
        QtMocHelpers::SlotData<void(size_t, size_t, size_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 }, { 0x80000000 | 3, 6 },
        }}),
        // Slot 'setXYYColumns'
        QtMocHelpers::SlotData<void(int, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'setYColumn2'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setYColumn2'
        QtMocHelpers::SlotData<void(size_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Slot 'setValue2Column'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'yColumn2'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYYGraph, qt_meta_tag_ZN13JKQTPXYYGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYYGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXYYGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXYYGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13JKQTPXYYGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXYYGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYYGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setXYYColumns((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<size_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<size_t>>(_a[3]))); break;
        case 1: _t->setXYYColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->setYColumn2((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setYColumn2((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setValue2Column((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getYColumn2(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setYColumn2(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXYYGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYYGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXYYGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXYGraph::qt_metacast(_clname);
}

int JKQTPXYYGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN13JKQTPXXYGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXXYGraph::qt_create_metaobjectdata<qt_meta_tag_ZN13JKQTPXXYGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXXYGraph",
        "setXXYColumns",
        "",
        "size_t",
        "xCol",
        "x2Col",
        "yCol",
        "setXColumn2",
        "__value",
        "setKey2Column",
        "xColumn2"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setXXYColumns'
        QtMocHelpers::SlotData<void(size_t, size_t, size_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 }, { 0x80000000 | 3, 6 },
        }}),
        // Slot 'setXXYColumns'
        QtMocHelpers::SlotData<void(int, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'setXColumn2'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setXColumn2'
        QtMocHelpers::SlotData<void(size_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Slot 'setKey2Column'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'xColumn2'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXXYGraph, qt_meta_tag_ZN13JKQTPXXYGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXXYGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXXYGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXXYGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13JKQTPXXYGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXXYGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXXYGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setXXYColumns((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<size_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<size_t>>(_a[3]))); break;
        case 1: _t->setXXYColumns((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->setXColumn2((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setXColumn2((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setKey2Column((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getXColumn2(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setXColumn2(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXXYGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXXYGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPXXYGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXYGraph::qt_metacast(_clname);
}

int JKQTPXXYGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPSingleColumnGraph::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPSingleColumnGraph",
        "setDataColumn",
        "",
        "__value",
        "size_t",
        "setDataDirection",
        "DataDirection",
        "setDataSortOrder",
        "DataSortOrder",
        "sortData",
        "dataColumn",
        "dataDirection",
        "Unsorted",
        "Sorted",
        "X",
        "Y"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setDataColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setDataColumn'
        QtMocHelpers::SlotData<void(size_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 3 },
        }}),
        // Slot 'setDataDirection'
        QtMocHelpers::SlotData<void(enum DataDirection)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 3 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(enum DataSortOrder)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sortData'
        QtMocHelpers::PropertyData<enum DataSortOrder>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'dataColumn'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'dataDirection'
        QtMocHelpers::PropertyData<enum DataDirection>(11, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DataSortOrder'
        QtMocHelpers::EnumData<enum DataSortOrder>(8, 8, QMC::EnumFlags{}).add({
            {   12, DataSortOrder::Unsorted },
            {   13, DataSortOrder::Sorted },
        }),
        // enum 'DataDirection'
        QtMocHelpers::EnumData<enum DataDirection>(6, 6, QMC::EnumIsScoped).add({
            {   14, DataDirection::X },
            {   15, DataDirection::Y },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPSingleColumnGraph, qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPSingleColumnGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPSingleColumnGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPSingleColumnGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setDataColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setDataColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 2: _t->setDataDirection((*reinterpret_cast<std::add_pointer_t<enum DataDirection>>(_a[1]))); break;
        case 3: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<enum DataSortOrder>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum DataSortOrder*>(_v) = _t->getDataSortOrder(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getDataColumn(); break;
        case 2: *reinterpret_cast<enum DataDirection*>(_v) = _t->getDataDirection(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataSortOrder(*reinterpret_cast<enum DataSortOrder*>(_v)); break;
        case 1: _t->setDataColumn(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setDataDirection(*reinterpret_cast<enum DataDirection*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPSingleColumnGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPSingleColumnGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPSingleColumnGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPSingleColumnGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYAndVectorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYAndVectorGraph",
        "setDxColumn",
        "",
        "col",
        "setDyColumn",
        "setDxDyColumn",
        "colDx",
        "colDy",
        "setAngleColumn",
        "setAngleAndLengthColumn",
        "colAngle",
        "colLength",
        "setLengthColumn",
        "vectorDataLayout",
        "VectorDataLayout",
        "dxColumn",
        "dyColumn",
        "angleColumn",
        "lengthColumn",
        "DeltaXDeltaYLayout",
        "AngleAndLengthLayout",
        "DefaultVectorDataLayout"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setDxColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setDyColumn'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setDxDyColumn'
        QtMocHelpers::SlotData<void(int, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'setAngleColumn'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setAngleAndLengthColumn'
        QtMocHelpers::SlotData<void(int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'setLengthColumn'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'vectorDataLayout'
        QtMocHelpers::PropertyData<enum VectorDataLayout>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'dxColumn'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'dyColumn'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'angleColumn'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'lengthColumn'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VectorDataLayout'
        QtMocHelpers::EnumData<enum VectorDataLayout>(14, 14, QMC::EnumFlags{}).add({
            {   19, VectorDataLayout::DeltaXDeltaYLayout },
            {   20, VectorDataLayout::AngleAndLengthLayout },
            {   21, VectorDataLayout::DefaultVectorDataLayout },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPXYAndVectorGraph, qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYAndVectorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXYAndVectorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYAndVectorGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setDxColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setDyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setDxDyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->setAngleColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setAngleAndLengthColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->setLengthColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum VectorDataLayout*>(_v) = _t->getVectorDataLayout(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getDxColumn(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getDyColumn(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->getAngleColumn(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->getLengthColumn(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setDxColumn(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setDyColumn(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setAngleColumn(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setLengthColumn(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXYAndVectorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYAndVectorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTPXYAndVectorGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXYGraph::qt_metacast(_clname);
}

int JKQTPXYAndVectorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN11JKQTPXGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXGraph::qt_create_metaobjectdata<qt_meta_tag_ZN11JKQTPXGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXGraph",
        "setDataSortOrder",
        "",
        "__value",
        "DataSortOrder",
        "setXColumn",
        "size_t",
        "setKeyColumn",
        "sortData",
        "xColumn",
        "Unsorted",
        "SortedX"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setDataSortOrder'
        QtMocHelpers::SlotData<void(enum DataSortOrder)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 3 },
        }}),
        // Slot 'setXColumn'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setXColumn'
        QtMocHelpers::SlotData<void(size_t)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 3 },
        }}),
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sortData'
        QtMocHelpers::PropertyData<enum DataSortOrder>(8, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'xColumn'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DataSortOrder'
        QtMocHelpers::EnumData<enum DataSortOrder>(4, 4, QMC::EnumFlags{}).add({
            {   10, DataSortOrder::Unsorted },
            {   11, DataSortOrder::SortedX },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPXGraph, qt_meta_tag_ZN11JKQTPXGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPXGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPXGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11JKQTPXGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setDataSortOrder((*reinterpret_cast<std::add_pointer_t<enum DataSortOrder>>(_a[1]))); break;
        case 2: _t->setXColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setXColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum DataSortOrder*>(_v) = _t->getDataSortOrder(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getXColumn(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDataSortOrder(*reinterpret_cast<enum DataSortOrder*>(_v)); break;
        case 1: _t->setXColumn(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *JKQTPXGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPXGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGraph::qt_metacast(_clname);
}

int JKQTPXGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
