/****************************************************************************
** Meta object code from reading C++ file 'jkqtpgeolines.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpgeolines.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpgeolines.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12JKQTPGeoLineE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoLine::qt_create_metaobjectdata<qt_meta_tag_ZN12JKQTPGeoLineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoLine"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoLine, qt_meta_tag_ZN12JKQTPGeoLineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoLine::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoBaseDecoratedLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPGeoLineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPGeoLineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12JKQTPGeoLineE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoLine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoLine *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoLine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoLine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12JKQTPGeoLineE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoBaseDecoratedLine::qt_metacast(_clname);
}

int JKQTPGeoLine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoBaseDecoratedLine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13JKQTPGeoArrowE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoArrow::qt_create_metaobjectdata<qt_meta_tag_ZN13JKQTPGeoArrowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoArrow"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoArrow, qt_meta_tag_ZN13JKQTPGeoArrowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoArrow::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPGeoArrowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPGeoArrowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13JKQTPGeoArrowE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoArrow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoArrow *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoArrow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoArrow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13JKQTPGeoArrowE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoLine::qt_metacast(_clname);
}

int JKQTPGeoArrow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoLine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoInfiniteLine::qt_create_metaobjectdata<qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoInfiniteLine"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoInfiniteLine, qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoInfiniteLine::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoBaseDecoratedHeadLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoInfiniteLine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoInfiniteLine *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoInfiniteLine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoInfiniteLine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20JKQTPGeoInfiniteLineE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoBaseDecoratedHeadLine::qt_metacast(_clname);
}

int JKQTPGeoInfiniteLine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoBaseDecoratedHeadLine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoPolyLines::qt_create_metaobjectdata<qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoPolyLines"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoPolyLines, qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoPolyLines::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoBaseDecoratedLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoPolyLines::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoPolyLines *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoPolyLines::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoPolyLines::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPGeoPolyLinesE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoBaseDecoratedLine::qt_metacast(_clname);
}

int JKQTPGeoPolyLines::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoBaseDecoratedLine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoBezierCurve::qt_create_metaobjectdata<qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoBezierCurve"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoBezierCurve, qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoBezierCurve::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoBaseDecoratedLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoBezierCurve::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoBezierCurve *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoBezierCurve::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoBezierCurve::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTPGeoBezierCurveE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoBaseDecoratedLine::qt_metacast(_clname);
}

int JKQTPGeoBezierCurve::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoBaseDecoratedLine::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11JKQTPGeoArcE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPGeoArc::qt_create_metaobjectdata<qt_meta_tag_ZN11JKQTPGeoArcE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPGeoArc"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPGeoArc, qt_meta_tag_ZN11JKQTPGeoArcE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPGeoArc::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPGeoBaseLine::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPGeoArcE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPGeoArcE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11JKQTPGeoArcE_t>.metaTypes,
    nullptr
} };

void JKQTPGeoArc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPGeoArc *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPGeoArc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPGeoArc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11JKQTPGeoArcE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPGeoBaseLine::qt_metacast(_clname);
}

int JKQTPGeoArc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPGeoBaseLine::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
