/****************************************************************************
** Meta object code from reading C++ file 'jkqtpimpulses.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpimpulses.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpimpulses.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImpulsesGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImpulsesGraphBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setDrawSymbols",
        "__value",
        "setDrawBaseline"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDrawSymbols'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawBaseline'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImpulsesGraphBase, qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImpulsesGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYBaselineGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPImpulsesGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImpulsesGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setDrawSymbols((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setDrawBaseline((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImpulsesGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImpulsesGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22JKQTPImpulsesGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineStyleMixin"))
        return static_cast< JKQTPGraphLineStyleMixin*>(this);
    if (!strcmp(_clname, "JKQTPGraphSymbolStyleMixin"))
        return static_cast< JKQTPGraphSymbolStyleMixin*>(this);
    return JKQTPXYBaselineGraph::qt_metacast(_clname);
}

int JKQTPImpulsesGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYBaselineGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImpulsesHorizontalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImpulsesHorizontalGraph",
        "setKeyColumn",
        "",
        "__value",
        "setValueColumn"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setKeyColumn'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'setValueColumn'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImpulsesHorizontalGraph, qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImpulsesHorizontalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImpulsesGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPImpulsesHorizontalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImpulsesHorizontalGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setKeyColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->setValueColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImpulsesHorizontalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImpulsesHorizontalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPImpulsesHorizontalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPImpulsesGraphBase::qt_metacast(_clname);
}

int JKQTPImpulsesHorizontalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImpulsesGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImpulsesHorizontalErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImpulsesHorizontalErrorGraph",
        "setErrorSymmetric",
        "",
        "__value",
        "setErrorStyle",
        "JKQTPErrorPlotstyle",
        "setErrorColumn",
        "column",
        "size_t",
        "setErrorColumnLower"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setErrorSymmetric'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setErrorStyle'
        QtMocHelpers::SlotData<void(JKQTPErrorPlotstyle)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
        // Slot 'setErrorColumn'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setErrorColumnLower'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setErrorColumnLower'
        QtMocHelpers::SlotData<void(size_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImpulsesHorizontalErrorGraph, qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImpulsesHorizontalErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImpulsesHorizontalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPImpulsesHorizontalErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImpulsesHorizontalErrorGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setErrorSymmetric((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setErrorStyle((*reinterpret_cast<std::add_pointer_t<JKQTPErrorPlotstyle>>(_a[1]))); break;
        case 2: _t->setErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setErrorColumnLower((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setErrorColumnLower((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImpulsesHorizontalErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImpulsesHorizontalErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTPImpulsesHorizontalErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPXGraphErrors"))
        return static_cast< JKQTPXGraphErrors*>(this);
    return JKQTPImpulsesHorizontalGraph::qt_metacast(_clname);
}

int JKQTPImpulsesHorizontalErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImpulsesHorizontalGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImpulsesVerticalGraph::qt_create_metaobjectdata<qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImpulsesVerticalGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImpulsesVerticalGraph, qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImpulsesVerticalGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImpulsesGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPImpulsesVerticalGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImpulsesVerticalGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPImpulsesVerticalGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImpulsesVerticalGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26JKQTPImpulsesVerticalGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPImpulsesGraphBase::qt_metacast(_clname);
}

int JKQTPImpulsesVerticalGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImpulsesGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPImpulsesVerticalErrorGraph::qt_create_metaobjectdata<qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPImpulsesVerticalErrorGraph",
        "setErrorSymmetric",
        "",
        "__value",
        "setErrorStyle",
        "JKQTPErrorPlotstyle",
        "setErrorColumn",
        "column",
        "size_t",
        "setErrorColumnLower"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setErrorSymmetric'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setErrorStyle'
        QtMocHelpers::SlotData<void(JKQTPErrorPlotstyle)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
        // Slot 'setErrorColumn'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setErrorColumn'
        QtMocHelpers::SlotData<void(size_t)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setErrorColumnLower'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setErrorColumnLower'
        QtMocHelpers::SlotData<void(size_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPImpulsesVerticalErrorGraph, qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPImpulsesVerticalErrorGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPImpulsesVerticalGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPImpulsesVerticalErrorGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPImpulsesVerticalErrorGraph *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setErrorSymmetric((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->setErrorStyle((*reinterpret_cast<std::add_pointer_t<JKQTPErrorPlotstyle>>(_a[1]))); break;
        case 2: _t->setErrorColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->setErrorColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 4: _t->setErrorColumnLower((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->setErrorColumnLower((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPImpulsesVerticalErrorGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPImpulsesVerticalErrorGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31JKQTPImpulsesVerticalErrorGraphE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPYGraphErrors"))
        return static_cast< JKQTPYGraphErrors*>(this);
    return JKQTPImpulsesVerticalGraph::qt_metacast(_clname);
}

int JKQTPImpulsesVerticalErrorGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPImpulsesVerticalGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
