/****************************************************************************
** Meta object code from reading C++ file 'jkqtpevaluatedparametriccurve.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpevaluatedparametriccurve.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpevaluatedparametriccurve.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYFunctionLineGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYFunctionLineGraphBase",
        "setTMin",
        "",
        "val",
        "setTMax",
        "setTRange",
        "tmin_",
        "tmax_",
        "std::pair<double,double>",
        "range"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setTMin'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setTMax'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setTRange'
        QtMocHelpers::SlotData<void(double, double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 }, { QMetaType::Double, 7 },
        }}),
        // Slot 'setTRange'
        QtMocHelpers::SlotData<void(const QPair<double,double> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYFunctionLineGraphBase, qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYFunctionLineGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPEvaluatedFunctionGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPXYFunctionLineGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYFunctionLineGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setTMin((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->setTMax((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setTRange((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 3: _t->setTRange((*reinterpret_cast<std::add_pointer_t<std::pair<double,double>>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPXYFunctionLineGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYFunctionLineGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28JKQTPXYFunctionLineGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineStyleMixin"))
        return static_cast< JKQTPGraphLineStyleMixin*>(this);
    return JKQTPEvaluatedFunctionGraphBase::qt_metacast(_clname);
}

int JKQTPXYFunctionLineGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPEvaluatedFunctionGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXYFunctionLineGraph::qt_create_metaobjectdata<qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXYFunctionLineGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXYFunctionLineGraph, qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXYFunctionLineGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYFunctionLineGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXYFunctionLineGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXYFunctionLineGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPXYFunctionLineGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXYFunctionLineGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24JKQTPXYFunctionLineGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPXYFunctionLineGraphBase::qt_metacast(_clname);
}

int JKQTPXYFunctionLineGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYFunctionLineGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
