/****************************************************************************
** Meta object code from reading C++ file 'jkqtpgraphsbasestyle.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/jkqtpgraphsbasestyle.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpgraphsbasestyle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFillStyleSummmary::qt_create_metaobjectdata<qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFillStyleSummmary"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFillStyleSummmary, qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFillStyleSummmary::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21JKQTFillStyleSummmaryE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTGraphsSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTGraphsSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTGraphsSpecificStyleProperties, qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTGraphsSpecificStyleProperties::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33JKQTGraphsSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTGeometricSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTGeometricSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTGeometricSpecificStyleProperties, qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTGeometricSpecificStyleProperties::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTGraphsSpecificStyleProperties>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36JKQTGeometricSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTAnnotationsSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTAnnotationsSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTAnnotationsSpecificStyleProperties, qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTAnnotationsSpecificStyleProperties::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTGeometricSpecificStyleProperties>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN38JKQTAnnotationsSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTBarchartSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTBarchartSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTBarchartSpecificStyleProperties, qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTBarchartSpecificStyleProperties::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTGraphsSpecificStyleProperties>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35JKQTBarchartSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTImpulseSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTImpulseSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTImpulseSpecificStyleProperties, qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTImpulseSpecificStyleProperties::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTGraphsSpecificStyleProperties>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34JKQTImpulseSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTFinancialSpecificStyleProperties::qt_create_metaobjectdata<qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTFinancialSpecificStyleProperties"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTFinancialSpecificStyleProperties, qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTFinancialSpecificStyleProperties::staticMetaObject = { {
    QtPrivate::MetaObjectForType<JKQTGraphsSpecificStyleProperties>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36JKQTFinancialSpecificStylePropertiesE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTGraphsBaseStyle::qt_create_metaobjectdata<qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTGraphsBaseStyle"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTGraphsBaseStyle, qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTGraphsBaseStyle::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19JKQTGraphsBaseStyleE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
