/****************************************************************************
** Meta object code from reading C++ file 'jkqtpevaluatedfunction.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpevaluatedfunction.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpevaluatedfunction.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_create_metaobjectdata<qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase",
        "setColor",
        "",
        "QColor",
        "c",
        "setDrawErrorPolygons",
        "__value",
        "setDrawErrorLines",
        "setErrorLineColor",
        "setErrorFillColor",
        "setErrorFillStyle",
        "Qt::BrushStyle",
        "setErrorLineStyle",
        "Qt::PenStyle",
        "setErrorLineWidth"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDrawErrorPolygons'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setDrawErrorLines'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setErrorLineColor'
        QtMocHelpers::SlotData<void(const QColor &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'setErrorFillColor'
        QtMocHelpers::SlotData<void(const QColor &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Slot 'setErrorFillStyle'
        QtMocHelpers::SlotData<void(Qt::BrushStyle)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 6 },
        }}),
        // Slot 'setErrorLineStyle'
        QtMocHelpers::SlotData<void(Qt::PenStyle)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 6 },
        }}),
        // Slot 'setErrorLineWidth'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase, qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPEvaluatedFunctionWithErrorsGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 1: _t->setDrawErrorPolygons((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->setDrawErrorLines((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->setErrorLineColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->setErrorFillColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->setErrorFillStyle((*reinterpret_cast<std::add_pointer_t<Qt::BrushStyle>>(_a[1]))); break;
        case 6: _t->setErrorLineStyle((*reinterpret_cast<std::add_pointer_t<Qt::PenStyle>>(_a[1]))); break;
        case 7: _t->setErrorLineWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN48JKQTPEvaluatedFunctionWithErrorsGraphDrawingBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineAndFillStyleMixin"))
        return static_cast< JKQTPGraphLineAndFillStyleMixin*>(this);
    return JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_metacast(_clname);
}

int JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPEvaluatedFunctionWithErrorsGraphBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPFunctorLineGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPFunctorLineGraphBase"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPFunctorLineGraphBase, qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPFunctorLineGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPFunctorLineGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPFunctorLineGraphBase *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPFunctorLineGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPFunctorLineGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25JKQTPFunctorLineGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_metacast(_clname);
}

int JKQTPFunctorLineGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPEvaluatedFunctionWithErrorsGraphDrawingBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPXFunctionLineGraph::qt_create_metaobjectdata<qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPXFunctionLineGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPXFunctionLineGraph, qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPXFunctionLineGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFunctorLineGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPXFunctionLineGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPXFunctionLineGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPXFunctionLineGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPXFunctionLineGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPXFunctionLineGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPFunctorLineGraphBase::qt_metacast(_clname);
}

int JKQTPXFunctionLineGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFunctorLineGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPYFunctionLineGraph::qt_create_metaobjectdata<qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPYFunctionLineGraph"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JKQTPYFunctionLineGraph, qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPYFunctionLineGraph::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPFunctorLineGraphBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>.metaTypes,
    nullptr
} };

void JKQTPYFunctionLineGraph::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPYFunctionLineGraph *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *JKQTPYFunctionLineGraph::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPYFunctionLineGraph::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23JKQTPYFunctionLineGraphE_t>.strings))
        return static_cast<void*>(this);
    return JKQTPFunctorLineGraphBase::qt_metacast(_clname);
}

int JKQTPYFunctionLineGraph::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPFunctorLineGraphBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
