/****************************************************************************
** Meta object code from reading C++ file 'jkqtpbarchartbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../JKQtPlotter/lib/jkqtplotter/graphs/jkqtpbarchartbase.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jkqtpbarchartbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17JKQTPBarGraphBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto JKQTPBarGraphBase::qt_create_metaobjectdata<qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JKQTPBarGraphBase",
        "setFillMode",
        "",
        "JKQTPBarGraphBase::FillMode",
        "mode",
        "setFillBrushFunctor",
        "JKQTPBarGraphBase::FillBrushFunctor",
        "f",
        "JKQTPBarGraphBase::FillBrushFunctor&&",
        "JKQTPBarGraphBase::SimpleFillBrushFunctor",
        "JKQTPBarGraphBase::SimpleFillBrushFunctor&&",
        "setCustomDrawingFunctor",
        "JKQTPBarGraphBase::CustomDrawingFunctor&&",
        "JKQTPBarGraphBase::CustomDrawingFunctor",
        "setUseCustomDrawFunctor",
        "enabled",
        "autoscaleBarWidthAndShift",
        "maxWidth",
        "shrinkFactor",
        "autoscaleBarWidthAndShiftSeparatedGroups",
        "groupWidth",
        "setShift",
        "__value",
        "setWidth",
        "setDrawBaseline",
        "setStackSeparation",
        "setRectRadiusAtValue",
        "setRectRadiusAtBaseline",
        "setRectRadius",
        "all",
        "atValue",
        "atBaseline",
        "setLineColorDerivationModeForSpecialFill",
        "JKQTPColorDerivationMode",
        "m",
        "setColor",
        "QColor",
        "c",
        "setBarPositionColumn",
        "column",
        "size_t",
        "setBarHeightColumn",
        "FillMode",
        "SingleFilling",
        "TwoColorFilling",
        "FunctorFilling"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setFillMode'
        QtMocHelpers::SlotData<void(JKQTPBarGraphBase::FillMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setFillBrushFunctor'
        QtMocHelpers::SlotData<void(const JKQTPBarGraphBase::FillBrushFunctor &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setFillBrushFunctor'
        QtMocHelpers::SlotData<void(JKQTPBarGraphBase::FillBrushFunctor &&)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 7 },
        }}),
        // Slot 'setFillBrushFunctor'
        QtMocHelpers::SlotData<void(const JKQTPBarGraphBase::SimpleFillBrushFunctor &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 7 },
        }}),
        // Slot 'setFillBrushFunctor'
        QtMocHelpers::SlotData<void(JKQTPBarGraphBase::SimpleFillBrushFunctor &&)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 7 },
        }}),
        // Slot 'setCustomDrawingFunctor'
        QtMocHelpers::SlotData<void(JKQTPBarGraphBase::CustomDrawingFunctor &&)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 7 },
        }}),
        // Slot 'setCustomDrawingFunctor'
        QtMocHelpers::SlotData<void(const JKQTPBarGraphBase::CustomDrawingFunctor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 7 },
        }}),
        // Slot 'setUseCustomDrawFunctor'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'autoscaleBarWidthAndShift'
        QtMocHelpers::SlotData<void(double, double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 17 }, { QMetaType::Double, 18 },
        }}),
        // Slot 'autoscaleBarWidthAndShift'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 17 },
        }}),
        // Slot 'autoscaleBarWidthAndShift'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'autoscaleBarWidthAndShiftSeparatedGroups'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 20 },
        }}),
        // Slot 'autoscaleBarWidthAndShiftSeparatedGroups'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setShift'
        QtMocHelpers::SlotData<void(double)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 },
        }}),
        // Slot 'setWidth'
        QtMocHelpers::SlotData<void(double)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 },
        }}),
        // Slot 'setDrawBaseline'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'setStackSeparation'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 },
        }}),
        // Slot 'setRectRadiusAtValue'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 },
        }}),
        // Slot 'setRectRadiusAtBaseline'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 22 },
        }}),
        // Slot 'setRectRadius'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 29 },
        }}),
        // Slot 'setRectRadius'
        QtMocHelpers::SlotData<void(double, double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 30 }, { QMetaType::Double, 31 },
        }}),
        // Slot 'setLineColorDerivationModeForSpecialFill'
        QtMocHelpers::SlotData<void(const JKQTPColorDerivationMode &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(QColor)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'setBarPositionColumn'
        QtMocHelpers::SlotData<void(int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'setBarPositionColumn'
        QtMocHelpers::SlotData<void(size_t)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 39 },
        }}),
        // Slot 'setBarHeightColumn'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'setBarHeightColumn'
        QtMocHelpers::SlotData<void(size_t)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FillMode'
        QtMocHelpers::EnumData<enum FillMode>(42, 42, QMC::EnumFlags{}).add({
            {   43, FillMode::SingleFilling },
            {   44, FillMode::TwoColorFilling },
            {   45, FillMode::FunctorFilling },
        }),
    };
    return QtMocHelpers::metaObjectData<JKQTPBarGraphBase, qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JKQTPBarGraphBase::staticMetaObject = { {
    QMetaObject::SuperData::link<JKQTPXYBaselineGraph::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>.metaTypes,
    nullptr
} };

void JKQTPBarGraphBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JKQTPBarGraphBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setFillMode((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::FillMode>>(_a[1]))); break;
        case 1: _t->setFillBrushFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::FillBrushFunctor>>(_a[1]))); break;
        case 2: _t->setFillBrushFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::FillBrushFunctor&&>>(_a[1]))); break;
        case 3: _t->setFillBrushFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::SimpleFillBrushFunctor>>(_a[1]))); break;
        case 4: _t->setFillBrushFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::SimpleFillBrushFunctor&&>>(_a[1]))); break;
        case 5: _t->setCustomDrawingFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::CustomDrawingFunctor&&>>(_a[1]))); break;
        case 6: _t->setCustomDrawingFunctor((*reinterpret_cast<std::add_pointer_t<JKQTPBarGraphBase::CustomDrawingFunctor>>(_a[1]))); break;
        case 7: _t->setUseCustomDrawFunctor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->autoscaleBarWidthAndShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 9: _t->autoscaleBarWidthAndShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->autoscaleBarWidthAndShift(); break;
        case 11: _t->autoscaleBarWidthAndShiftSeparatedGroups((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->autoscaleBarWidthAndShiftSeparatedGroups(); break;
        case 13: _t->setShift((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->setWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->setDrawBaseline((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setStackSeparation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->setRectRadiusAtValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->setRectRadiusAtBaseline((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->setRectRadius((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->setRectRadius((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 21: _t->setLineColorDerivationModeForSpecialFill((*reinterpret_cast<std::add_pointer_t<JKQTPColorDerivationMode>>(_a[1]))); break;
        case 22: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 23: _t->setBarPositionColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->setBarPositionColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        case 25: _t->setBarHeightColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->setBarHeightColumn((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *JKQTPBarGraphBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JKQTPBarGraphBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17JKQTPBarGraphBaseE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "JKQTPGraphLineStyleMixin"))
        return static_cast< JKQTPGraphLineStyleMixin*>(this);
    if (!strcmp(_clname, "JKQTPGraphFillStyleMixin"))
        return static_cast< JKQTPGraphFillStyleMixin*>(this);
    return JKQTPXYBaselineGraph::qt_metacast(_clname);
}

int JKQTPBarGraphBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = JKQTPXYBaselineGraph::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
